import ipaddress
import socket
import struct
from abc import ABC
from collections import OrderedDict

from ipscap.util.raw_socket_entity import IPHeader, ICMPHeader, TCPHeader, UDPHeader


class IPHeaderParser:
    def __init__(self, eth_ips=None):
        self.eth_ips = eth_ips

    def initialize(self, eth_ips):
        self.eth_ips = eth_ips

    def get_ip_header(self, data):
        return data[0:20]

    def parse(self, mtu_data, raws=False):
        ip_header = self.create_ip_header()

        ip_header.header_data = self.get_ip_header(mtu_data)

        iph = struct.unpack('!BBHHHBBH4s4s', ip_header.header_data)

        version_ihl = iph[0]

        ip_header.version = version_ihl >> 4

        ihl = version_ihl & 0xF

        ip_header.iph_length = ihl * 4

        ip_header.tos = iph[1]
        ip_header.total_length = iph[2]
        ip_header.identification = iph[3]

        flags_fragment_offset = iph[4]
        ip_header.flags = (flags_fragment_offset >> 13)
        ip_header.fragment_offset = flags_fragment_offset & 0x1FFF

        ip_header.ttl = iph[5]
        ip_header.protocol = iph[6]
        ip_header.protocol_code = IPHeader.get_protocol_code(ip_header.protocol)

        ip_header.checksum = iph[7]

        ip_header.src_ip = socket.inet_ntoa(iph[8])
        ip_header.src_ip_int = int.from_bytes(iph[8], byteorder='big')

        ip_header.dest_ip = socket.inet_ntoa(iph[9])
        ip_header.dest_ip_int = int.from_bytes(iph[9], byteorder='big')

        ip_header.direction = self.detect_direction(ip_header)
        ip_header.direction_code = IPHeader.get_direction_code(ip_header.direction)

        if raws:
            ip_header.raws = self.parse_bytes(ip_header)

        return ip_header

    def parse_bytes(self, ip_header):
        raws = {}

        hdata = ip_header.header_data

        raws['version_ihl'] = self.get_hex(hdata, 0)
        raws['total_length'] = self.get_hex(hdata, 2, 2)
        raws['identification'] = self.get_hex(hdata, 4, 2)
        raws['flags_fragment_offset'] = self.get_hex(hdata, 6, 2)
        raws['ttl'] = self.get_hex(hdata, 8)
        raws['protocol'] = self.get_hex(hdata, 9)
        raws['checksum'] = self.get_hex(hdata, 10, 2)
        raws['src_ip'] = self.get_hex(hdata, 12, 4)
        raws['dest_ip'] = self.get_hex(hdata, 16, 4)

        return raws

    def get_hex(self, header_data, begin, size=1):
        end = begin + size

        return header_data[begin:end].hex().upper()

    def detect_direction(self, ip_header):
        direction = 0

        if ip_header.src_ip_int in self.eth_ips:
            direction = IPHeader.DIRECTION_SEND
        elif ip_header.dest_ip_int in self.eth_ips:
            direction = IPHeader.DIRECTION_RECEIVE
        else:
            tip = ipaddress.ip_address(ip_header.src_ip)

            if tip.is_private:
                direction = IPHeader.DIRECTION_SEND

            tip = ipaddress.ip_address(ip_header.dest_ip)

            if tip.is_private:
                direction = IPHeader.DIRECTION_RECEIVE

        return direction

    def get_header_parser(self, ip_header):
        if ip_header.protocol == IPHeader.PROTOCOL_ICMP:
            parser = self.create_icmp_header_parser()
        elif ip_header.protocol == IPHeader.PROTOCOL_TCP:
            parser = self.create_tcp_header_parser()
        elif ip_header.protocol == IPHeader.PROTOCOL_UDP:
            parser = self.create_udp_header_parser()
        else:
            raise Exception('Unknown Parser')

        return parser

    def create_ip_header(self):
        return IPHeader()

    def create_icmp_header_parser(self):
        return ICMPHeaderParser()

    def create_tcp_header_parser(self):
        return TCPHeaderParser()

    def create_udp_header_parser(self):
        return UDPHeaderParser()


class HeaderParser(ABC):
    def parse(self, ip_header, data, raws=False):
        pass

    def get_hex(self, header_data, begin, size=1):
        end = begin + size

        return header_data[begin:end].hex()


class ICMPHeaderParser(HeaderParser):
    def create_icmp_header(self):
        return ICMPHeader()

    def parse(self, ip_header, mtu_data, raws=False):
        icmp_header = self.create_icmp_header()

        icmp_header.icmph_length = ICMPHeader.DEFAULT_HEADER_LEN
        icmph_start = ip_header.iph_length
        icmp_header.header_data = mtu_data[icmph_start:icmph_start + ICMPHeader.DEFAULT_HEADER_LEN]

        icmph = struct.unpack('!BBH', icmp_header.header_data)

        icmp_header.icmp_type = icmph[0]
        icmp_header.code = icmph[1]
        icmp_header.checksum = icmph[2]

        if raws:
            icmp_header.raws = self.parse_bytes(icmp_header)

        return icmp_header

    def parse_bytes(self, icmp_header):
        raws = {}

        hdata = icmp_header.header_data

        raws['icmp_type'] = self.get_hex(hdata, 0)
        raws['code'] = self.get_hex(hdata, 1)
        raws['checksum'] = self.get_hex(hdata, 2, 2)

        return raws


class TCPHeaderParser(HeaderParser):
    def create_tcp_header(self):
        return TCPHeader()

    def parse(self, ip_header, mtu_data, raws=False):
        tcp_header = self.create_tcp_header()

        tcph_start = ip_header.iph_length
        tcph_data = mtu_data[tcph_start:tcph_start + TCPHeader.DEFAULT_HEADER_LEN]

        tcph = struct.unpack('!HHLLHHHH', tcph_data)

        tcp_header.src_port = tcph[0]
        tcp_header.dest_port = tcph[1]
        tcp_header.seq_no = tcph[2]
        tcp_header.ack_no = tcph[3]

        offset_flags = tcph[4]

        tcp_header.tcph_length = (offset_flags >> 12) * 4
        tcp_header.flags = offset_flags & 0x1FF

        tcp_header.flag_codes = TCPHeader.get_flag_codes(tcp_header.flags)

        tcp_header.window = tcph[5]
        tcp_header.checksum = tcph[6]
        tcp_header.urgent_pointer = tcph[7]

        payload_start = ip_header.iph_length + tcp_header.tcph_length

        tcp_header.header_data = mtu_data[tcph_start:payload_start]

        options_data = tcp_header.header_data[TCPHeader.DEFAULT_HEADER_LEN:]
        tcp_header.tcp_options = self.parse_tcp_options(options_data)

        tcp_header.payload_data = mtu_data[payload_start:]
        tcp_header.payload_length = len(tcp_header.payload_data)

        if raws:
            tcp_header.raws = self.parse_bytes(tcp_header)

        return tcp_header

    def parse_tcp_options(self, data):
        pos = 0
        tcp_options = OrderedDict()

        while pos < len(data):
            kind = data[pos]

            if kind == 0:  # End of Option List
                break

            elif kind == 1:  # NOP
                tcp_options['nop'] = True
                pos += 1

            elif kind == 2:  # MSS
                tcp_options['mss'] = struct.unpack('!H', data[pos + 2:pos + 4])[0]
                pos += 4

            elif kind == 3:  # WScale
                tcp_options['wscale'] = struct.unpack('!B', data[pos + 2:pos + 3])[0]
                pos += 3

            elif kind == 4:  # SACK
                tcp_options['sack'] = True
                pos += 2

            elif kind == 8:  # Timestamp
                tsval, tsecr = struct.unpack('!II', data[pos + 2:pos + 10])
                tcp_options['timestamp'] = tsval
                pos += 10

            else:
                if pos + 1 >= len(data):
                    break

                length = data[pos + 1]

                if length < 2:
                    break

                pos += length

        return tcp_options

    def parse_bytes(self, tcp_header):
        raws = {}

        hdata = tcp_header.header_data

        raws['src_port'] = self.get_hex(hdata, 0, 2)
        raws['dest_port'] = self.get_hex(hdata, 2, 2)
        raws['seq_no'] = self.get_hex(hdata, 4, 4)
        raws['ack_no'] = self.get_hex(hdata, 8, 4)
        raws['flags'] = self.get_hex(hdata, 12, 2)
        raws['window'] = self.get_hex(hdata, 14, 2)
        raws['checksum'] = self.get_hex(hdata, 16, 2)
        raws['urgent_pointer'] = self.get_hex(hdata, 18, 2)

        return raws


class UDPHeaderParser(HeaderParser):
    def create_udp_header(self):
        return UDPHeader()

    def parse(self, ip_header, mtu_data, raws=False):
        udp_header = self.create_udp_header()

        udp_header.udph_length = UDPHeader.DEFAULT_HEADER_LEN
        udph_start = ip_header.iph_length
        udph_data = mtu_data[udph_start:udph_start + UDPHeader.DEFAULT_HEADER_LEN]

        udph = struct.unpack('!HHHH', udph_data)

        udp_header.src_port = udph[0]
        udp_header.dest_port = udph[1]
        udp_header.udp_length = udph[2]
        udp_header.checksum = udph[3]

        payload_start = ip_header.iph_length + 8

        udp_header.header_data = udph_data

        udp_header.payload_data = mtu_data[payload_start:udph_start + udp_header.udp_length]
        udp_header.payload_length = len(udp_header.payload_data)

        if raws:
            udp_header.raws = self.parse_bytes(udp_header)

        return udp_header

    def parse_bytes(self, udp_header):
        raws = {}

        hdata = udp_header.header_data

        raws['src_port'] = self.get_hex(hdata, 0, 2)
        raws['dest_port'] = self.get_hex(hdata, 2, 2)
        raws['udph_length'] = self.get_hex(hdata, 4, 2)
        raws['checksum'] = self.get_hex(hdata, 6, 2)

        return raws



import re

from ipsurv.core.entity import ValueData, StoreBase
from ipsurv.serializer.serializer import Serializer
from ipsurv.util.sys_util import AppException


class LineSerializer(Serializer):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#serializer
    """

    def transform(self, data):
        super().transform(data)

        self.transform_status(data)

    def transform_status(self, data):
        # type: (ValueData) -> None
        """
        :param data:
        :type data: ValueData
        """

        if data.get('in_range') is not None:
            data.update('in_range', lambda v: 'RANGE_OK' if v else 'RANGE_NG')

        if data.get('icmp') is not None:
            data.update('icmp', lambda v: 'ICMP_OK' if v else 'ICMP_NG')

        if data.get('tcp') is not None:
            data.update('tcp', lambda v: 'TCP_OK' if v else 'TCP_NG')

        if data.get('udp') is not None:
            data.update('udp', lambda v: 'UDP_OK' if v else 'UDP_NG')

        if data.get('http') is not None:
            data.update('http', lambda v: 'HTTP_OK' if v else 'HTTP_NG')

        if data.get('http_h2') is not None:
            data.update('http_h2', lambda v: 'HTTP2' if v == 1 else 'HTTP1' if v == 0 else 'N/A')

    def filter_value(self, v):
        if v is not None:
            if isinstance(v, StoreBase):
                return v
            elif isinstance(v, bool):
                v = self._get_bool_status(v)

            v = str(v)

            if self.alt_delimiter and not self.enclose:
                delimiter = re.escape(self.delimiter)
                v = re.sub(delimiter, self.alt_delimiter, v)
        else:
            v = ''

        return self._append_enclose(v)

    def _append_enclose(self, v):
        enclose = self.enclose

        v = re.sub(re.escape(enclose), enclose + enclose, v)

        return enclose + v + enclose

    def _get_bool_status(self, v):
        return 'OK' if v else 'NG'

    def build_row(self, data):
        values = data.get_data()

        try:
            line = self.format.format(**values)
        except Exception:
            raise AppException('Format error.')

        return line

    def build_error(self, error):
        line = 'ERROR' + self.delimiter + error

        return line

    def output(self, v):
        print(v, flush=True)

    def transform_key_labels(self, data, mode):
        # type: (ValueData, int) -> None

        values = data.get_data()

        item = {}

        for k, v in values.items():
            k = self.get_label(k, mode)
            item[k] = v

        data.set_data(item)

    def output_message(self, msg):
        self.output(msg)

    def output_item(self, data):
        values = data.get_data()

        for name, value in values.items():
            self.output(name + self.delimiter + str(value))

        self.output('')



from ipsend.configs import Constant
from ipsend.util.data_io import DataInput
from ipsend.util.raw_socket_entity import TCPHeader
import re


class PacketGenerator:
    def __init__(self, factory, data_input):
        self.factory = factory
        self.data_input = data_input

    def create_data(self, args):
        byte_data = None

        data = self.data_input.get_data(args.data)

        if args.mode == Constant.MODE_ICMP_PAYLOAD:
            generator, entity = self.factory.create_ip_header_generator()
            byte_data = self._create_ip_header_data(generator, entity, args, data)
        elif args.mode == Constant.MODE_TCP_PAYLOAD:
            generator, entity = self.factory.create_tcp_header_generator()
            byte_data = self._create_tcp_header_data(generator, entity, args, data)
        elif args.mode == Constant.MODE_UDP_PAYLOAD:
            generator, entity = self.factory.create_udp_header_generator()
            byte_data = self._create_udp_header_data(generator, entity, args, data)
        elif args.mode == Constant.MODE_ICMP_PAYLOAD:
            generator, entity = self.factory.create_icmp_header_generator()
            byte_data = self._create_icmp_header_data(generator, entity, args, data)
        else:
            raise Exception('Unknown mode.')

        self.data_input.initialize(DataInput.INPUT_RAW)

        return byte_data

    def _create_ip_header_data(self, generator, entity, args, data):
        entity.version = 4
        entity.iph_length = 0
        entity.tos = 0
        entity.total_length = 0
        entity.identification = args.ip_identification
        entity.flags = args.ip_flags
        entity.fragment_offset = 0
        entity.ttl = args.ip_ttl
        entity.protocol = args.ip_protocol
        entity.src_ip = args.src_ip
        entity.src_ip_int = None
        entity.dest_ip = args.dest_ip
        entity.dest_ip_int = None
        entity.payload_data = data

        return generator.generate(entity, args.src_ip, args.dest_ip)

    def _create_tcp_header_data(self, generator, entity, args, data):
        entity.src_port = args.src_port
        entity.dest_port = args.dest_port

        entity.seq_no = args.tcp_seq
        entity.ack_no = args.tcp_ack

        entity.flags = TCPHeader.get_flags(re.split(r'[;,\s]+', args.tcp_flags))
        entity.window = args.tcp_window
        entity.tcp_options = 0
        entity.payload_data = data

        return generator.generate(entity, args.src_ip, args.dest_ip)

    def _create_udp_header_data(self, generator, entity, args, data):
        entity.src_port = args.src_port
        entity.dest_port = args.dest_port
        entity.payload_data = data

        return generator.generate(entity, args.src_ip, args.dest_ip)

    def _create_icmp_header_data(self, generator, entity, args, data):
        entity.type = args.icmp_type
        entity.code = args.icmp_code
        entity.identifier = args.icmp_id
        entity.sequence = args.icmp_seq
        entity.payload_data = data

        return generator.generate(entity, args.src_ip, args.dest_ip)



import ipaddress
import pytest

from ipsurv.core.entity import Target, ValueData
from ipsurv.core.target_parser import TargetParser
from ipsurv.core.pipeline import Pipeline
from ipsurv.requester.dns_resolver import DnsResolveRequester
from unittest.mock import MagicMock


class TestTargetParser:
    @pytest.fixture(autouse=True)
    def setup(self, args, dns_resolver):
        pipeline = Pipeline()

        self.target_parser = TargetParser(args, pipeline, dns_resolver)

    @pytest.fixture
    def args(self, mocker):
        args = mocker.Mock()
        args.fixed_format_params = []
        args.fixed_timeout = {'dns': 0, 'http': 0, 'reactivity': 0}
        args.fixed_delimiter = ','
        args.fixed_collectors = []
        args.fixed_ranges = []
        args.autodetect = False

        return args

    @pytest.fixture
    def dns_resolver(self, mocker):
        dns_resolver = mocker.Mock(spec=DnsResolveRequester)

        dns_resolver.resolve_ip = MagicMock(return_value='192.168.1.200')

        return dns_resolver

    def test_parse(self, args):
        data = ValueData({})

        self.target_parser.parse(data, 'a,192.168.1.100,b', args)

        assert data.get('identifier') == '192.168.1.100'
        assert data.get('identifier_int') == 3232235876

        target = self.target_parser.parse(data, 'a,x,b', args)

        assert target.identifier == 'ILLEGAL_FORMAT'

    def test_parse_target(self, args):
        data = ValueData({})

        target = self.target_parser._parse_target(data, '192.168.1.100', args)

        assert target.raw == '192.168.1.100'

        self.target_parser.autodetect = True
        target = self.target_parser._parse_target(data, '192.168.1.100', args)

        assert target.raw == '192.168.1.100'

    def test_identify_target(self, args):
        data = ValueData({})

        target = Target('192.168.1.100')

        self.target_parser._identify_target(data, target, args)

        assert data.get('ip') == '192.168.1.100'
        assert data.get('ip_int') == 3232235876

        target = Target('')

        self.target_parser._identify_target(data, target, args)

        assert target.status == 'EMPTY'

    def test_identify_target_ip(self, args):
        data = ValueData({})

        target = Target('192.168.1.100')

        identified = self.target_parser._identify_target_ip(data, target, args)

        assert identified is True

        target = Target('http://www.wikipedia.org/')

        identified = self.target_parser._identify_target_ip(data, target, args)

        assert identified is True

        target = Target('192.168.1.10 - - [10/Nov/2024:03:15:42 +0900] "GET / HTTP/1.1" 304 - "http://xyz-sample-dummy-test.org" "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36"')

        identified = self.target_parser._identify_target_ip(data, target, args)

        assert identified is True
        assert target.ip == '192.168.1.10'

    def test_assign_data_target(self):
        data = ValueData({})

        target = Target()
        target.identifier = '192.168.1.100'
        target.identifier_int = 3232235876

        self.target_parser._assign_data_target(data, target)

        assert data.get('identifier') == '192.168.1.100'
        assert data.get('identifier_int') == 3232235876

    def test_detect_target_raw(self):
        target = Target()
        target.identifier = 'test'
        target.identifier_int = 2730633496

        data = ValueData({})

        self.target_parser._assign_data_target(data, target)

        assert data.get('identifier') == 'test'
        assert data.get('identifier_int') == 2730633496
        assert type(data.get('target')) == Target

    def test_split_port(self):
        v1, v2 = self.target_parser._split_port('ipsurv-domain-test-example.xyz:80')

        assert v1 == 'ipsurv-domain-test-example.xyz' and v2 == 80

        v1, v2 = self.target_parser._split_port('192.168.1.100:80')

        assert v1 == '192.168.1.100' and v2 == 80

        v1, v2 = self.target_parser._split_port('192.168.1.100')

        assert v1 == '192.168.1.100' and v2 is None

    def test_evaluate_ip_type(self):
        data = ValueData({})

        ip_address = ipaddress.ip_address('192.168.1.100')

        self.target_parser._evaluate_ip_type(data, ip_address)

        assert data.get('ip_type') == 1

        ip_address = ipaddress.ip_address('8.8.8.8')

        self.target_parser._evaluate_ip_type(data, ip_address)

        assert data.get('ip_type') == 2

    def test_evaluate_in_ranges(self):
        data = ValueData({})

        ip_address = ipaddress.ip_address('192.168.1.100')
        self.target_parser.ranges = ['192.168.1.1/24']

        self.target_parser._evaluate_in_ranges(data, ip_address)

        assert data.get('in_range') is True

    def test_find_url(self):
        assert self.target_parser._find_url('http://ipsurv-domain-test-example.xyz') is not None
        assert self.target_parser._find_url('https://ipsurv-domain-test-example.xyz') is not None
        assert self.target_parser._find_url('//ipsurv-domain-test-example.xyz') is not None

    def test_find_fqdn(self):
        assert self.target_parser._find_fqdn('ipsurv-domain-test-example.xyz') is not None
        assert self.target_parser._find_fqdn('test.psurv-domain-test-example.xyz') is not None

    def test_find_ip(self):
        assert self.target_parser._find_ip('192.168.1.1') is not None

    def test_create_target(self):
        assert type(self.target_parser._create_target('')) == Target



import fcntl
import ipaddress
import socket
import struct
from ipsurv.util.sys_util import AppException


class EthSocket:
    ETH_LEN = 14

    def __init__(self):
        self.sock = None

    def create_socket(self):
        try:
            self.sock = socket.socket(socket.AF_PACKET, socket.SOCK_RAW, socket.htons(3))
        except PermissionError:
            raise AppException('Permission error. Please run as "root" user.')

    def get_eth_ips(self):
        ips = []

        for interface in socket.if_nameindex():
            name = interface[1]
            ips.append(self.get_eth_ip(name))

        return ips

    def get_eth_ips_int(self):
        return list(map(lambda v: int(ipaddress.ip_address(v)), self.get_eth_ips()))

    def get_eth_ip(self, ifname):
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        addr = fcntl.ioctl(
            s.fileno(),
            0x8915,  # SIOCSIFADDR
            struct.pack('256s', ifname.encode('utf-8'))
        )

        return socket.inet_ntoa(addr[20:24])

    def recvfrom(self, bufsize):
        data, addr = self.sock.recvfrom(bufsize)

        return data

    def get_eth_header(self, data):
        return data[:self.ETH_LEN]

    def is_enabled_protocol(self, eth_header):
        eth = struct.unpack('!6s6sH', eth_header)
        eth_protocol = socket.ntohs(eth[2])

        # IP Protocol number = 8: IPv4
        if eth_protocol == 8:
            return True

        return False

    def get_ip_mtu(self, data):
        return data[self.ETH_LEN:]

    def __del__(self):
        if self.sock:
            self.sock.close()



import pprint
import logging
import os
import sys
import platform


class AppException(Exception):
    pass


class System:
    @classmethod
    def get_python_ver(cls):
        major = sys.version_info.major
        minor = sys.version_info.minor

        return float(major) + minor / 10

    @classmethod
    def verify_os(cls, windows=False, macos=False, linux=False):
        os_name = platform.system()

        if windows and os_name == 'Windows':
            return True

        if macos and os_name == 'Darwin':
            return True

        if linux and os_name == 'Linux':
            return True

        return False

    @classmethod
    def load_module(cls, name):
        module = None

        try:
            module = __import__(name)
        except ImportError:
            pass

        return module

    @classmethod
    def is_logging(cls, min_level=logging.INFO):
        level = cls.get_log_level()

        if level > 0 and level <= min_level:
            return True

        return False

    @classmethod
    def get_log_level(cls):
        logger = logging.getLogger()
        level = logger.getEffectiveLevel()

        return level

    @classmethod
    def output_data(cls, title, data, level=logging.INFO, indent=2):
        formatted_data = pprint.pformat(data, indent=indent)
        cls.output_body(title, formatted_data, level=level)

    @classmethod
    def output_body(cls, title, data, level=logging.INFO):
        logging.log(level, title + ':\n' + data)

    @classmethod
    def line(cls, msg):
        print(msg, flush=True)

    @classmethod
    def warn(cls, msg):
        print('\033[33m' + msg + '\033[0m', flush=True)

    @classmethod
    def exit(cls, msg, error=False):
        if error is False:
            cls.line(msg)
        else:
            cls.warn(msg)

        os._exit(error)



from ipsurv.data_collector.data_collector import DataCollector
from ipsurv.util.sys_util import System
import logging


class GeoIpCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.geoip.GeoIpRequester
    :param args:
    :type args: argparse.Namespace

    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/using_geoip2.html
    """

    COUNTRY_COLS = ['continent', 'continent_name', 'country', 'country_name']
    CITY_COLS = ['continent', 'continent_name', 'country', 'country_name', 'subdivision', 'subdivision_name', 'city', 'city_name', 'timezone', 'geo']
    ASN_COLS = ['asn', 'org']

    def __init__(self, requester, args):
        super().__init__(requester, args)

    def initialize(self, args):
        geoip_conf = args.conf.get('geoip')

        path = None
        files = None

        if isinstance(geoip_conf, dict):
            path = geoip_conf.get('path')
            ts = geoip_conf.get('files')
            files = ts if isinstance(ts, dict) else None

        self.requester.initialize(path, files)

        if self.requester.geoip2 is not None:
            if System.is_logging():
                logging.log(logging.INFO, 'GEOIP:ENABLED')
                logging.log(logging.INFO, 'GEOIP_DATA_PATH:' + self.requester.data_path)
                logging.log(logging.INFO, 'GEOIP_DATA_FILES:' + str(self.requester.data_files))
        else:
            logging.log(logging.INFO, 'GEOIP:DISABLED')

    def get_name(self):
        return 'GEOIP'

    def request_data(self, target, requires):
        ip = target.ip

        error_city = None
        error_country = None
        response = {'types': []}

        only_city_cols = set(self.CITY_COLS) - set(self.COUNTRY_COLS)
        require_only_city = True if requires is not None and only_city_cols & set(requires) else False

        if requires is None or set(self.CITY_COLS) & set(requires):
            error_city = self._request_city(ip, response)

        if requires is None or set(self.COUNTRY_COLS) & set(requires) and not response.get('country'):
            error_country = self._request_country(ip, response)

        if require_only_city and error_city:
            raise error_city
        elif error_country:
            raise error_country

        if requires is None or set(self.ASN_COLS) & set(requires):
            self._request_asn(ip, response)

        return True, response

    def _request_city(self, ip, response):
        error = None

        try:
            response['types'].append('city')

            res = self.requester.request_city(ip)

            response.update(res)
        except Exception as e:
            error = e

        return error

    def _request_country(self, ip, response):
        error = None

        try:
            response['types'].append('country')

            res = self.requester.request_country(ip)

            response.update(res)
        except Exception as e:
            error = e

        return error

    def _request_asn(self, ip, response):
        response['types'].append('asn')

        res = self.requester.request_asn(ip)

        response.update(res)

    def get_requires(self):
        return ['continent', 'continent_name', 'country', 'country_name', 'city', 'city_name', 'geo', 'org', 'asn', 'timezone']

    def build_data(self, target, data, success, response, response_time):
        data.set('geoip_time', response_time)

        self.put(data, response, 'continent')
        self.put(data, response, 'continent_name')
        self.put(data, response, 'subdivision')
        self.put(data, response, 'subdivision_name')
        self.put(data, response, 'country')
        self.put(data, response, 'country_name')
        self.put(data, response, 'city')
        self.put(data, response, 'city_name')
        self.put(data, response, 'organization', 'org')
        self.put(data, response, 'asn')
        self.put(data, response, 'timezone')

        if response.get('latitude') is not None and response.get('longitude') is not None:
            geo = str(response.get('latitude')) + ',' + str(response.get('longitude'))
            data.set('geo', geo)

        self._append_error(data, response)



from abc import ABC

from ipsend.configs import Config, Constant


class Pipeline(ABC):
    """

    """

    def __init__(self):
        self.config = None  # type: Config

    def initialize(self, config):
        """
        :param config:
        :type config: Config
        """

        self.config = config

    def init_configure(self, arguments):
        """
        :param arguments:
        :type arguments: dict
        """
        pass

    def complete(self):
        pass



import logging

from ipsurv.configs import Config, Constant
from ipsurv.core.pipeline import Pipeline
from ipsurv.survey_ips import SurveyIps
from ipsurv.survey_self import SurveySelf
from ipsurv.util.sys_util import System
from ipsurv import __version__


class IpSurvCmd:
    def __init__(self, factory):
        self.factory = factory
        self.pipeline = self.factory.create_pipeline()  # type: Pipeline
        self.config = self.factory.get_config()  # type: Config

    def run(self):
        args, rows = self._parse_args()

        if args.version:
            self._output_version()

        serializer = self.factory.create_serializer(args)

        self.pipeline.initialize(self.config, serializer)

        data_factory = self.factory.create_value_data_factory(args, self.config)

        num = len(rows)
        mode = self._detect_survey_mode(num, rows)

        self.pipeline.begin_process(mode, args, rows)

        if num >= 1:
            if mode == Constant.MODE_SURVEY_IPS:
                self._survey_ips(args, data_factory, serializer, rows)
            else:
                self._survey_self(args, data_factory, serializer)
        else:
            System.warn('No target data.')

        self.pipeline.complete_process(mode, args, rows)

    def _detect_survey_mode(self, num, rows):
        if num == 1 and 'self' in rows[0].lower():
            mode = Constant.MODE_SURVEY_SELF
        else:
            mode = Constant.MODE_SURVEY_IPS

        return self.pipeline.detect_survey_mode(mode)

    def _parse_args(self):
        args_builder = self.factory.create_args_builder(self.config, self.pipeline)

        args = args_builder.parse()

        rows = args_builder.read_lines(args)

        return args, rows

    def _survey_ips(self, args, data_factory, serializer, rows):
        logging.log(logging.INFO, 'MODE:SURVEY_IPS')

        survey_ips = self._build(args, data_factory, serializer)

        survey_ips.initialize(args)

        survey_ips.dispatch(rows, args)

    def _build(self, args, data_factory, serializer):
        dns_resolver = self.factory.create_dns_resolver(args)

        target_parser = self.factory.create_target_parser(args, self.pipeline, dns_resolver)

        _collectors = self.factory.create_collectors(args, dns_resolver)

        collectors = [_collectors[key] for key in args.fixed_collectors]

        reactivities = self.factory.create_reactivities(args)

        return SurveyIps(args, self.config, data_factory, target_parser, collectors, reactivities, self.pipeline, serializer)

    def _survey_self(self, args, data_factory, serializer):
        logging.log(logging.INFO, 'MODE:SURVEY_SELF')

        dns_resolver = self.factory.create_dns_resolver(args)

        server_reactivity = self.factory.create_server_reactivity(args)

        collector = self.factory.create_self_collector(args, dns_resolver, server_reactivity)

        survey_self = SurveySelf(args, self.config, data_factory, collector, self.pipeline, serializer)

        survey_self.initialize()

        survey_self.dispatch()

    def _output_version(self):
        System.exit(Constant.APP_NAME + ' ' + __version__)



import pytest

from ipsurv.configs import Constant, Config


class TestConstant:
    def test_vars(self):
        assert Constant.APP_NAME == 'ipsurv'
        assert hasattr(Constant, 'APP_DESCRIPTION')
        assert hasattr(Constant, 'APP_BOTTOM_DESC')
        assert hasattr(Constant, 'ENV_ARGS_VAR')
        assert hasattr(Constant, 'ENV_CONF_VAR')
        assert hasattr(Constant, 'DELIMITER_DEFAULT')


class TestConfig:
    def test_vars(self):
        assert hasattr(Config, 'PRE_ARGUMENTS')
        assert hasattr(Config, 'ARGUMENTS')
        assert hasattr(Config, 'ENV_CONFS')
        assert hasattr(Config, 'FORMAT_PROFILES')
        assert hasattr(Config, 'FORMAT_PARAMS')
        assert hasattr(Config, 'MASTER_DATA')
        assert hasattr(Config, 'COLLECTORS')
        assert hasattr(Config, 'HEAD_MSG_SELF')



import dns.resolver
import ipaddress

from ipsurv.requester.requester import Requester
from ipsurv.util.network_util import DnsUtil


class DnsResolveRequester(Requester):
    """
    :param resolver:
    :type resolver: dns.resolver
    :param timeout:
    :type timeout: float

    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#requester
    """

    def __init__(self, resolver=None, timeout=4.0):
        super().__init__(timeout)

        self.resolver = resolver

    def request_resolve(self, hostname):
        success = False
        response = {}

        ip = self.resolve_ip(hostname)

        if ip:
            success = True
            response = {'ip': ip}

        return success, response

    def resolve_ip(self, hostname, port=None):
        ip = DnsUtil.resolve(hostname, port, timeout=self.timeout)

        return ip

    def request_reverse(self, ip):
        hostname = DnsUtil.reverse(ip, timeout=self.timeout)

        response = {'hostname': hostname}
        success = True

        return success, response

    # "15169 | 8.8.8.0/24 | US | arin | 2023-12-28"
    def request_dnstxt(self, ip):
        server = 'origin.asn.cymru.com'

        resolver = self.get_resolver()

        resolver.lifetime = self.timeout

        reversed_ip = str(dns.reversename.from_address(ip, v4_origin=None))

        v = reversed_ip + '.' + server

        tv = resolver.query(v, 'TXT')

        vals = str(tv[0]).strip("\"'\t ").split('|')

        vals = list(map(lambda v: v.strip(), vals))

        cidr = vals[1]

        network = ipaddress.ip_network(cidr, strict=False)

        response = {
            'cidr': cidr,
            'network_start': str(network.network_address),
            'network_end': str(network.broadcast_address),
            'country': vals[2],
            'rir': vals[3],
            'date': vals[4]
        }

        success = True

        return success, response

    def get_resolver(self):
        if self.resolver is None:
            self.resolver = dns.resolver.Resolver()
            self.resolver.timeout = self.timeout

        return self.resolver



from abc import ABC

from ipsurv.configs import Config, Constant
from ipsurv.core.entity import Target, TargetGroup, ValueData
from ipsurv.data_collector.data_collector import DataCollector
from ipsurv.serializer.serializer import Serializer


class Pipeline(ABC):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#pipeline
    """
    def __init__(self):
        self.config = None  # type: Config
        self.serializer = None  # type: Serializer

    def initialize(self, config, serializer):
        """
        :param config:
        :type config: Config
        :param serializer:
        :type serializer: Serializer
        """

        self.config = config
        self.serializer = serializer

    def init_configure(self, arguments, env_args):
        """
        :param arguments:
        :type arguments: dict
        :param env_args:
        :type env_args: dict
        """
        pass

    def pre_configure(self, args, env_args, env_conf):
        """
        :param args:
        :type args: argparse.Namespace
        :param env_args:
        :type env_args: dict
        :param env_conf:
        :type env_conf: dict
        """
        pass

    def post_configure(self, args, env_args, env_conf):
        """
        :param args:
        :type args: argparse.Namespace
        :param env_args:
        :type env_args: dict
        :param env_conf:
        :type env_conf: dict
        """
        pass

    def detect_survey_mode(self, mode):
        """
        :param mode:
        :type mode: int
        """
        self.serializer.set_survey_mode(mode)

        return mode

    def begin_process(self, mode, args, rows):
        """
        :param mode:
        :type mode: int
        :param args:
        :type args: argparse.Namespace
        :param rows:
        :type rows: list
        """

        self.serializer.output_begin(mode, args, rows)

    def pre_target_parse(self, data, original):
        """
        :param data:
        :type data: ValueData
        :param original:
        :type original: str
        """

        return original

    def pre_target_identify(self, data, target):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        """

        return True

    def pre_output_headers(self, data):
        """
        :param data:
        :type data: ValueData
        """

        pass

    def pre_collect(self, data, target, args):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        :param args:
        :type args: argparse.Namespace
        """

        pass

    def find_group(self, data, target):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        :rtype: TargetGroup
        """

        return None

    def get_group_identify(self, data, target):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        :rtype: int
        """

        return target.identifier_int

    def create_group(self, data, target, group_type, cidr):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        :param group_type:
        :type group_type: int
        :param cidr:
        :type cidr: str
        :rtype: TargetGroup
        """

        return None

    def pre_request(self, data, name, collector):
        """
        :param data:
        :type data: ValueData
        :param name:
        :type name: str
        :param collector:
        :type collector: DataCollector
        """

        pass

    def post_request(self, data, name, collector, success, response):
        """
        :param data:
        :type data: ValueData
        :param name:
        :type name: str
        :param collector:
        :type collector: DataCollector
        :param success:
        :type success: bool
        :param response:
        :type response: dict
        """

        pass

    def post_collect(self, data, target, args, skip):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        :param args:
        :type args: argparse.Namespace
        :param skip:
        :type skip: bool
        """

        self.serializer.set_status(data, target, args, skip)

    def build(self, data, target):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        """

        return self.serializer.build(data, target)

    def build_error(self, error):
        return self.serializer.build_error(error)

    def output_result(self, v):
        self.serializer.output_result(v)

    def output_result_self(self, data):
        """
        :param data:
        :type data: ValueData
        """

        self.serializer.transform_key_labels(data, Constant.STR_PASCAL)

        self.serializer.output_item(data)

    def complete_process(self, mode, args, rows):
        """
        :param mode:
        :type mode: int
        :param args:
        :type args: argparse.Namespace
        :param rows:
        :type rows: list
        """

        self.serializer.output_complete(mode, args, rows)



import argparse
import json
import logging
import os
import sys
from abc import ABC, abstractmethod
import select


class StrAction(argparse.Action):
    def __call__(self, parser, namespace, v, option_string=None):
        setattr(namespace, self.dest, v.replace('\\t', '\t'))


class ArgValidator(ABC):
    def __init__(self, debug=False):
        self.name = None

        self.debug = debug
        self.error = 'Argument value error.'

    def validate(self, args):
        try:
            r = self._validate(args)
        except Exception as e:
            if isinstance(e, argparse.ArgumentError) or self.debug:
                raise e
            else:
                raise self.arg_error(self.error)

        return r

    @abstractmethod
    def _validate(self, args):  # pragma: no cover
        return None

    def arg_error(self, msg):
        e = argparse.ArgumentError(None, msg)

        e.argument_name = '--' + self.name

        return e


class ArgsHelper:
    @staticmethod
    def init_parser(arguments, formatter_class=argparse.ArgumentDefaultsHelpFormatter, raw=False, group_names=None):
        parser = argparse.ArgumentParser(add_help=False, formatter_class=formatter_class)

        ArgsHelper.add_arguments(parser, arguments, [], raw=raw, group_names=group_names)

        args, unknown = parser.parse_known_args()

        return parser, args

    @staticmethod
    def init_logging(verbose, log):
        if verbose > 0:
            if verbose == 1:
                level = logging.ERROR
            elif verbose == 2:
                level = logging.INFO
            else:
                level = logging.DEBUG
        else:
            level = logging.CRITICAL

        opts = {
            'level': level,
            'format': '%(asctime)s - %(levelname)s - %(message)s'
        }

        if log is None:
            opts['stream'] = sys.stdout
        else:
            opts['filename'] = log

        logging.basicConfig(**opts)

    @staticmethod
    def add_arguments(parser, arguments, overrides, raw=False, group_names=None):
        groups = {}

        for arg, options in arguments.items():
            if raw:
                options = {'action': options['action']} if 'action' in options else {}

            if 'group' in options:
                group_key = options['group']

                if group_key not in groups:
                    name = group_names[group_key] if group_names is not None and group_key in group_names else group_key
                    target_parser = parser.add_argument_group(name)
                    groups[group_key] = target_parser
                else:
                    target_parser = groups[group_key]

                del options['group']
            else:
                target_parser = parser

            if arg in overrides:
                options['default'] = overrides[arg]

            if options.get('action') == 'StrAction':
                options['action'] = StrAction

            if 'type' in options and 'choices' not in options and 'metavar' not in options:
                options['metavar'] = ''

            if not options.get('nargs'):
                params = []

                format = '--{}'

                if 'shorten' in options:
                    if options['shorten'] is not True:
                        params.append(options['shorten'])
                    else:
                        format = '-{}'

                    del options['shorten']

                params.append(format.format(arg))

                target_parser.add_argument(*params, **options)
            else:
                target_parser.add_argument(arg, **options)


class StdinLoader:
    @staticmethod
    def read_stdin(timeout=2.0):
        r, _, _ = select.select([sys.stdin], [], [], timeout)

        if r:
            input_data = sys.stdin.read()
            lines = input_data.splitlines()
        else:
            lines = []

        return lines

    @staticmethod
    def load_env(name):
        error = False

        env = {}

        v = os.getenv(name)

        if v is not None and v != '':
            try:
                tv = json.loads(v)

                if type(env) is dict:
                    env = tv
                else:
                    error = True

            except Exception:
                logging.log(logging.DEBUG, 'Env parse error.(' + name + ')', exc_info=True)
                error = True

        if error:
            logging.log(logging.INFO, 'Fail to load env.(' + name + ')')

        return env



import pytest

from ipsurv.requester.ip_info import IpInfoRequester
from ipsurv.requester.rdap import RdapRequester, CountryDetector
from ipsurv.requester.dns_resolver import DnsResolveRequester
from ipsurv.requester.http import HttpRequester
from ipsurv.requester.server_reactivity import ServerReactivity
import http.client
import socket
import re


class TestRequester:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_create_http_connection(self):
        requester = IpInfoRequester()

        con = requester._create_http_connection('8.8.8.8')
        assert type(con).__name__ == 'HTTPSConnection'

        con = requester._create_http_connection('8.8.8.8', False)
        assert type(con).__name__ == 'HTTPConnection'


class TestIpInfoRequester:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self):
        requester = IpInfoRequester()

        success, response = requester.request('8.8.8.8')

        assert success is True
        assert response['ip'] == '8.8.8.8'
        assert response['country'] == 'US'
        assert response['timezone'] == 'America/Los_Angeles'

    def test_request_fail(self):
        requester = IpInfoRequester()

        with pytest.raises(http.client.HTTPException):
            success, response = requester.request('499.499.499.499')

    def test_timeout(self):
        requester = IpInfoRequester(timeout=0.005)

        with pytest.raises(socket.timeout):
            success, response = requester.request('53.112.1.0')


class TestIpRdapRequester:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self):
        country_detector = CountryDetector()
        requester = RdapRequester(country_detector)

        success, response = requester.request('8.8.8.8')

        assert success is True
        assert response['port43'] == 'whois.arin.net'
        assert response['country'] == 'US'

    def test_detect_server_from_ip(self):
        country_detector = CountryDetector()
        requester = RdapRequester(country_detector)

        server = requester.detect_server_from_ip('213.1.1.1')

        assert server is not None

    def test_get_octet2_by_ip(self):
        country_detector = CountryDetector()
        requester = RdapRequester(country_detector)

        v = requester.detect_server_from_ip('192.168.1.1')

        assert v == 'https://rdap.arin.net/registry/'

    def test_get_id_from_server(self):
        country_detector = CountryDetector()
        requester = RdapRequester(country_detector)

        v = requester.get_id_from_server('https://rdap.arin.net/registry/')

        assert v == 1

    def test_request_fail(self):
        country_detector = CountryDetector()
        requester = RdapRequester(country_detector)

        with pytest.raises(http.client.HTTPException):
            success, response = requester.request('499.499.499.499')

    def test_timeout(self):
        country_detector = CountryDetector()
        requester = RdapRequester(country_detector, timeout=0.001)

        with pytest.raises(Exception):
            success, response = requester.request('53.10.1.0')

    def test_request_http(self):
        country_detector = CountryDetector()
        requester = RdapRequester(country_detector)

        res, body = requester.request_http('http://google.com/')

        assert res is not None


class TestCountryDetector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_detect(self):
        country_detector = CountryDetector()

        code = country_detector.detect_by_address('119 Washington Square N, New York, NY 20011, USA 1100')
        assert code == 'US'

        code = country_detector.detect_by_address('119 Washington Square N, New York, NY 20011, US,')
        assert code == 'US'

        code = country_detector.detect_by_address('119 Washington Square N, New York, NY 20011, U.S.,')
        assert code == 'US'

        code = country_detector.detect_by_address('U.S., 219 Washington Square N, New York, NY 30011')
        assert code == 'US'

        code = country_detector.detect_by_address("Washington, 1475 Kind Plaza Ld, United States")
        assert code == 'US'

        code = country_detector.detect_by_address('142 ABC 6-ro, Sejong-si, 8932, Republic of Korea.')
        assert code == 'KR'

        code = country_detector.detect_by_address('Seoul support Center 1136 Sejong-daero, Jung-gu Seoul 804520, Korea')
        assert code == 'KR'

        code = country_detector.detect_by_address('Seoul support Center 1136 Sejong-daero, Jung-gu Seoul 804520, South Korea')
        assert code == 'KR'

        code = country_detector.detect_by_address("PO Box 86827 98721 Pyongyang Democratic People's Republic of Korea")
        assert code == 'KP'

        code = country_detector.detect_by_address("Jongno-gu, Republic of Korea")
        assert code == 'KR'

        code = country_detector.detect_by_address("Case postale 8330 CH-11218 Le Grand-A, Geneva, Switzerland")
        assert code == 'CH'

        code = country_detector.detect_by_address("Roosevelt Road, Taipei, 1870319 Taiwan")
        assert code == 'TW'

        code = country_detector.detect_by_address("Badu Rd., Zhongshan Dist., Taipei City 19704, Taiwan (R.O.C.)")
        assert code == 'TW'

        code = country_detector.detect_by_address('28-12-36 Uchi-Kanda;Chiyoda-ku;Tokyo 101-0047;japan')
        assert code == 'JP'

        code = country_detector.detect_by_address("Shad / Famine\n1875 rue d'Ague\n982100 Billancourt\nFr")
        assert code == 'FR'


class TestDnsResolveRequester:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request_resolve(self):
        requester = DnsResolveRequester()

        success, response = requester.request_resolve('google.com')

        assert re.match(r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', response['ip'])

        with pytest.raises(Exception):
            success, response = requester.request_resolve('dummy')

    def test_resolve_ip(self):
        requester = DnsResolveRequester()

        ip = requester.resolve_ip('google.com')

        assert re.match(r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', ip)

    def test_request_reverse(self):
        requester = DnsResolveRequester()

        success, response = requester.request_reverse('8.8.8.8')

        assert response['hostname'] == 'dns.google'

    def test_request_dnstxt(self):
        requester = DnsResolveRequester()

        success, response = requester.request_dnstxt('8.8.8.8')

        assert response['country'] == 'US'
        assert response['cidr'] == '8.8.8.0/24'

    def test_get_resolver(self):
        requester = DnsResolveRequester()

        assert type(requester.get_resolver()).__name__ == 'Resolver'

        requester = DnsResolveRequester({})

        assert type(requester.get_resolver()) == dict


class TestHttpRequester:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self):
        requester = HttpRequester()

        success, response = requester.request('wikipedia.org')

        assert success is True
        assert response['http_status'] == 200
        assert response['http_size'] > 256

        with pytest.raises(Exception):
            requester.request('wikipedia-not-found.xyz')

    def test_request_http(self):
        requester = HttpRequester()

        requester.set_headers({})

        res, body = requester.request_http('https://www.wikipedia.org')

        assert res.status == 200
        assert len(body) > 256

    def test_request_alpn_h2request_http(self):
        requester = HttpRequester()

        r = requester.request_alpn_h2('https://www.wikipedia.org')

        assert r == 1

        assert requester.request_alpn_h2('https://www.wikipedia-not-found.xyz') == -1

    def test_create_url(self):
        requester = HttpRequester()

        url = requester._create_url('//www.wikipedia.org')
        assert re.search(r'http:', url)

        url = requester._create_url('www.wikipedia.org')
        assert re.search(r'http:', url)

        url = requester._create_url('http://www.wikipedia.org')
        assert re.search(r'http:', url)


class TestServerReactivity:
    @pytest.fixture(autouse=True)
    def setup(self):
        self.requester = ServerReactivity()

    def test_request_icmp(self, monkeypatch):
        monkeypatch.setattr('subprocess.check_output', lambda v, universal_newlines=True: 0)
        success = self.requester.request_icmp('wikipedia.org')

        assert success is True

    def test_request_tcp(self):
        success = self.requester.request_tcpport('wikipedia.org', 80)

        assert success is True

        success = self.requester.request_tcpport('wikipedia.org', 443)

        assert success is True

    def test_request_udp(self):
        success = self.requester.request_udpport('8.8.8.8', 53)

        assert success is True

    def test_request_fail(self):
        requester = ServerReactivity(timeout=0.001)

        with pytest.raises(Exception):
            requester.request_icmp('6.1.1.1')

        with pytest.raises(Exception):
            requester.request_tcpport('wikipedia.org', 81)

        with pytest.raises(Exception):
            requester.request_udpport('8.8.8.8', 52)



import logging
import socket
import ssl
import threading

from ipsend.configs import Constant
from ipsurv.util.sys_util import AppException


class Socket:
    def __init__(self):
        self._sock = None
        self._data_input = None
        self._data_output = None
        self._mode = None
        self._timeout = -1

    @staticmethod
    def get_error(e):
        error = None

        if isinstance(e, socket.timeout):
            error = 'Connection timeout.'
        elif isinstance(e, socket.gaierror):
            error = 'Connection error.'
        elif isinstance(e, socket.herror) or isinstance(e, socket.error):
            error = 'Socket error.'

        return error

    def initialize(self, data_input, data_output, mode, timeout):
        pass

    def create(self, hostname, port=0):
        pass

    def connected(self):
        return (self._sock)

    def send(self, data):
        pass

    def receive(self, bufsize=Constant.RECV_BUF_SIZE):
        pass

    def close(self):
        if self._sock:
            self._sock.close()

            self._sock = None
            self._hostname = None
            self._port = None

    def __del__(self):
        self.close()


class RichSocket(Socket):
    def __init__(self):
        super().__init__()

    def initialize(self, data_input, data_output, mode, timeout):
        self._data_input = data_input
        self._data_output = data_output
        self._mode = mode
        self._timeout = timeout
        self._ssl_context = None

    def set_ssl_context(self, ssl_context):
        self._ssl_context = ssl_context

    def create(self, hostname, port=0):
        if self._mode != Constant.MODE_UDP:
            self._sock = self.create_tcp_socket(hostname, port)
        else:
            self._sock = self.create_udp_socket()

        self._hostname = hostname
        self._port = port

    def create_tcp_socket(self, hostname, port):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        sock.settimeout(self._timeout)

        sock.connect((hostname, port))

        if self._mode == Constant.MODE_SSL:
            if not self._ssl_context:
                context = ssl.create_default_context()
            else:
                context = ssl.SSLContext(self._ssl_context)

            sock = context.wrap_socket(sock, server_hostname=hostname)

        return sock

    def create_udp_socket(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        sock.settimeout(self._timeout)

        return sock

    def send(self, data):
        byte_data = self._data_input.get_data(data)

        if self._mode:
            self._sock.sendall(byte_data)
        else:
            self._sock.sendto(byte_data, (self._hostname, self._port))

    def receive(self, bufsize=Constant.RECV_BUF_SIZE):
        while True:
            if self._mode:
                data = self._sock.recv(bufsize)
            else:
                data, _ = self._sock.recvfrom(bufsize)

            if not data:
                break

            self._data_output.output_binary(data)

        self.close()


class RawSocket(Socket):
    def __init__(self):
        super().__init__()

        self._mode = None

    def initialize(self, data_input, data_output, mode, timeout):
        self._data_input = data_input
        self._data_output = data_output
        self._mode = mode
        self._timeout = timeout

    def connected(self):
        return (self._sock)

    def create(self, hostname, port=0):
        try:
            self._sock = socket.socket(socket.AF_INET, socket.SOCK_RAW, self.get_proto(self._mode))

            self._sock.settimeout(self._timeout)
        except PermissionError:
            raise AppException('Permission error. Please run as "root" user.')

        self._hostname = hostname

    def get_proto(self, mode):
        if mode in [Constant.MODE_IP_HEADER, Constant.MODE_IP_PAYLOAD]:
            proto = socket.IPPROTO_RAW
        elif mode in [Constant.MODE_TCP_HEADER, Constant.MODE_TCP_PAYLOAD]:
            proto = socket.IPPROTO_TCP
        elif mode == [Constant.MODE_UDP_HEADER, Constant.MODE_UDP_PAYLOAD]:
            proto = socket.IPPROTO_UDP
        elif mode == [Constant.MODE_ICMP_HEADER, Constant.MODE_ICMP_PAYLOAD]:
            proto = socket.IPPROTO_ICMP
        else:
            raise Exception('Unknown mode.')

        return proto

    def send(self, data):
        byte_data = self._data_input.get_data(data)

        self._sock.sendto(byte_data, (self._hostname, 0))

    def receive(self, bufsize=Constant.RECV_BUF_SIZE):
        while True:
            data, _ = self._sock.recvfrom(bufsize)

            self._data_output.output_binary(data)

        self.close()


class SocketThread(threading.Thread):
    def __init__(self, sock, view_helper):
        super().__init__()

        self.daemon = True

        self.sock = sock
        self.view_helper = view_helper

    def run(self):
        try:
            self.sock.receive()
        except Exception as e:
            self.sock.close()

            self.view_helper.output_error(e)

            logging.log(logging.ERROR, str(e), exc_info=True)



import re

from ipsurv.data_collector.data_collector import DataCollector
from ipsurv.util.network_util import DnsUtil


class RdapCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.rdap.RdapRequester
    :param args:
    :type args: argparse.Namespace
    """

    def __init__(self, requester, args):
        super().__init__(requester, args)

        self.dns_timeout = args.fixed_timeout['dns']

    def get_name(self):
        return 'RDAP'

    def request_data(self, target, requires):
        DnsUtil.resolve(self.requester.get_host(), timeout=self.dns_timeout)

        return self.requester.request(target.ip)

    def get_requires(self):
        return ['cidr', 'network_start', 'network_end', 'country', 'name', 'handle', 'org', 'address', 'port43', 'description']

    def get_cidr(self, response):
        return response.get('cidr')

    def build_data(self, target, data, success, response, response_time):
        data.set('rdap_time', response_time)

        self.put(data, response, 'cidr')
        self.put(data, response, 'startAddress', 'network_start')
        self.put(data, response, 'endAddress', 'network_end')
        self.put(data, response, 'country')
        self.put(data, response, 'country_updated')
        self.put(data, response, 'name')
        self.put(data, response, 'port43')

        if response.get('primary_entity'):
            entity = response.get('primary_entity')

            data.set('handle', entity.get('handle'))
            data.set('org', entity.get('name'))
            data.set('address', self.sanitize(entity.get('address')))

        description = ''

        if 'remarks' in response:
            remarks = response['remarks']

            if len(remarks) > 0 and 'description' in remarks[0] and len(remarks[0]['description']) > 0:
                description = self.sanitize(remarks[0]['description'][0])

        data.set('description', description)

        self._append_error(data, response)

    def sanitize(self, v):
        return re.sub(r'\s+', ' ', v)


class DnsTxtCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.dns_resolver.DnsResolveRequester
    :param args:
    :type args: argparse.Namespace
    """

    def get_name(self):
        return 'DNSTXT'

    def request_data(self, target, requires):
        return self.requester.request_dnstxt(target.ip)

    def get_requires(self):
        return ['cidr', 'network_start', 'network_end', 'country', 'rir']

    def get_cidr(self, response):
        return response.get('cidr')

    def build_data(self, target, data, success, response, response_time):
        data.set('dnstxt_time', response_time)

        self.put(data, response, 'cidr')
        self.put(data, response, 'network_start')
        self.put(data, response, 'network_end')
        self.put(data, response, 'country')
        self.put(data, response, 'rir')

        self._append_error(data, response)


class IpInfoCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.ip_info.IpInfoRequester
    :param args:
    :type args: argparse.Namespace
    """

    def __init__(self, requester, args):
        super().__init__(requester, args)

        self.dns_timeout = args.fixed_timeout['dns']

    def get_name(self):
        return 'IPINFO'

    def request_data(self, target, requires):
        DnsUtil.resolve(self.requester.get_host(), timeout=self.dns_timeout)

        return self.requester.request(target.ip)

    def get_requires(self):
        return ['ip', 'hostname', 'country', 'region_name', 'city_name', 'postal', 'geo', 'asn', 'org', 'timezone']

    def build_data(self, target, data, success, response, response_time):
        data.set('ipinfo_time', response_time)

        self.fill(data, response, 'ip')
        self.fill(data, response, 'hostname')
        self.put(data, response, 'country')
        self.put(data, response, 'city', 'city_name')
        self.put(data, response, 'region', 'region_name')
        self.put(data, response, 'postal')
        self.put(data, response, 'loc', 'geo')
        self.put(data, response, 'asn')
        self.put(data, response, 'org')
        self.put(data, response, 'timezone')

        self._append_error(data, response)


class DnsReverseCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.dns_resolver.DnsResolveRequester
    :param args:
    :type args: argparse.Namespace
    """

    def get_name(self):
        return 'DNSREVERSE'

    def request_data(self, target, requires):
        return self.requester.request_reverse(target.ip)

    def get_requires(self):
        return ['hostname']

    def build_data(self, target, data, success, response, response_time):
        data.set('dnsreverse_time', response_time)

        self.put(data, response, 'hostname')

        self._append_error(data, response)



import argparse
import logging
import re

from ipscap.configs import Constant
from ipscap.core.pipeline import Pipeline
from ipscap.util.raw_socket_entity import IPHeader
from ipsurv.util.args_util import ArgsHelper
from ipsurv.util.sys_util import System
from ipscap.util.evaluation_parser import EvaluationParser, EvaluationParserException
from ipsurv.util.sys_util import AppException


class ArgsBuilder:
    def __init__(self, config, pipeline, ev_parser):
        self.config = config
        self.pipeline = pipeline  # type: Pipeline
        self.ev_parser = ev_parser  # type: EvaluationParser

    def parse(self):
        parent_parser, args = self.init_args(self.config.PRE_ARGUMENTS)

        return self.build_args(parent_parser, self.config.ARGUMENTS)

    def init_args(self, arguments):
        parser, args = ArgsHelper.init_parser(arguments)

        if args.debug:
            args.verbose = 3

        ArgsHelper.init_logging(args.verbose, args.log)

        if args.verbose > 0:
            System.warn('Enable verbose mode. Current:' + str(args.verbose) + ' [Level - 1:TRACE_ERROR, 2:INFO, 3:DEBUG]')

            if args.log is not None:
                System.warn('Enable log.(File:' + args.log + ')')

        return parser, args

    def build_args(self, parent_parser, arguments):
        desc = self._create_bottom_desc()

        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter, parents=[parent_parser], description=Constant.APP_DESCRIPTION, epilog=desc)

        self.pipeline.init_configure(arguments)

        self._prepare_arguments(parser, arguments)

        args = parser.parse_args()

        self._assign_shorten_option(args)

        if System.is_logging():
            self.logging(args)

        self._configure(parser, args)

        self._parse_condition(args)

        self._notice(args)

        return (args, parser, self.ev_parser)

    def _create_bottom_desc(self):
        desc = ''

        desc += Constant.APP_BOTTOM_DESC + "\n"

        return desc

    def _assign_shorten_option(self, args):
        if args.web_port:
            args.port = '80,443,53'

        if args.general_port:
            args.port = '21,22,23,25,53,80,110,143,220,443,465,990,993,995,1433,3306'

        if args.exclude_ssh:
            if not args.condition:
                args.condition = ''

            args.condition = 'port!=22;' + args.condition

    def _configure(self, parser, args):
        try:
            args.fixed_find_mode = self._fix_find_mode(args)
            args.fixed_protocols = self.fix_protocols(args)
            args.fixed_ips = self._fix_ips(args)
            args.fixed_ports = self._fix_ports(args)
            args.fixed_output = self._fix_output(args)
            args.has_filters = self._has_filters(args)
        except Exception as e:
            logging.log(logging.DEBUG, 'Fix arguments error.', exc_info=True)

            parser.error(e)

    def _prepare_arguments(self, parser, arguments):
        ArgsHelper.add_arguments(parser, arguments, {})

    def _fix_find_mode(self, args):
        find_mode = args.find_mode.upper()

        defines = {
            1: Constant.FIND_REGEX,
            2: Constant.FIND_MATCH,
            3: Constant.FIND_BINARY,
            4: Constant.FIND_HEX,
        }

        if find_mode.isdigit():
            v = int(find_mode)

            if v in defines:
                find_mode = defines[v]
            else:
                find_mode = None
        else:
            if find_mode not in defines.values():
                find_mode = None

        if find_mode is None:
            raise Exception('Unknown output mode (--find_mode)')

        logging.log(logging.INFO, 'Fixed find_mode:' + find_mode)

        return find_mode

    def _fix_ips(self, args):
        ips = re.split(r'[;, ]+', args.ip)

        ips = list(filter(lambda v: v.strip(), ips))

        ips = ips if len(ips) > 0 else None

        logging.log(logging.INFO, 'Fixed ips:' + str(ips))

        return ips

    def _fix_ports(self, args):
        ports = re.split(r'[;, ]+', args.port)

        ports = list(filter(lambda v: v.strip(), ports))
        ports = list(map(lambda v: int(v), ports))

        ports = ports if len(ports) > 0 else None

        logging.log(logging.INFO, 'Fixed ports:' + str(ports))

        return ports

    def fix_protocols(self, args):
        protocols = []

        tprotocols = re.split(r'[;, ]+', args.protocol)

        for protocol in tprotocols:
            protocol_code = protocol.upper()

            if protocol_code == 'ALL':
                protocols = [IPHeader.PROTOCOL_ICMP, IPHeader.PROTOCOL_TCP, IPHeader.PROTOCOL_UDP]
                break

            protocol = IPHeader.get_protocol(protocol_code)

            if protocol > 0:
                protocols.append(protocol)
            else:
                raise Exception('Unknown protocol (--protocol)')

        logging.log(logging.INFO, 'Fixed protocols:' + str(protocols))

        return protocols

    def _fix_output(self, args):
        output = args.output.upper()

        defines = {
            0: Constant.OUTPUT_NONE,
            1: Constant.OUTPUT_HEADER,
            2: Constant.OUTPUT_TEXT,
            3: Constant.OUTPUT_BINARY,
            4: Constant.OUTPUT_BINARY_ALL,
            5: Constant.OUTPUT_HEX,
            6: Constant.OUTPUT_HEX_ALL,
            7: Constant.OUTPUT_BASE64,
            8: Constant.OUTPUT_BASE64_ALL,
            9: Constant.OUTPUT_LINE,
        }

        if output.isdigit():
            v = int(output)

            if v in defines:
                output = defines[v]
            else:
                output = None
        else:
            if output not in defines.values():
                output = None

        if output is None:
            raise Exception('Unknown output mode (--output)')

        logging.log(logging.INFO, 'Fixed show:' + output)

        return output

    def _has_filters(self, args):
        if IPHeader.PROTOCOL_TCP not in args.fixed_protocols:
            return True

        if args.find:
            return True

        if args.fixed_ips:
            return True

        if args.fixed_ports:
            return True

        if args.condition:
            return True

        return False

    def _parse_condition(self, args):
        self.ev_parser.initialize(self.config.CONDITION_RULES)

        try:
            parsed_cond = self.ev_parser.parse(args.condition)
        except EvaluationParserException as e:
            raise AppException(str(e))

        System.output_data('PARSED_CONDITION', parsed_cond)

    def logging(self, args):
        params = vars(args)

        System.output_data('ARGUMENTS', params)

    def _notice(self, args):
        pass



from abc import ABC, abstractmethod

from ipsurv.configs import Constant
from ipsurv.core.entity import Target
from ipsurv.core.entity import ValueData


class Serializer(ABC):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#serializer
    """

    def __init__(self, args):
        self.format = args.fixed_format
        self.delimiter = args.fixed_delimiter
        self.alt_delimiter = args.alt_delimiter
        self.enclose = args.fixed_enclose

        self.survey_mode = None

    def set_survey_mode(self, survey_mode):
        """
        :param survey_mode:
        :type survey_mode: int
        """

        self.survey_mode = survey_mode

    def set_delimiter(self, delimiter):
        """
        :param delimiter:
        :type delimiter: str
        """

        self.delimiter = delimiter

    def output_begin(self, mode, args, rows):
        """
        :param data:
        :type data: ValueData
        :param original:
        :type original: str
        """

        pass

    def create_labels(self, columns, mode):
        """
        :param columns:
        :type columns: dict
        :param mode:
        :type mode: int
        :rtype: dict
        """

        labels = {}

        for v in columns:
            labels[v] = self.get_label(v, mode)

        return labels

    def get_label(self, v, mode):
        if mode == Constant.STR_LOWER:
            return v
        elif mode == Constant.STR_PASCAL:
            ts = v.split('_')
            return ts[0].capitalize() + ''.join(w.capitalize() for w in ts[1:])

        return v.upper()

    def set_status(self, data, target, args, skip):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        :param args:
        :type args: argparse.Namespace
        :param skip:
        :type skip: bool
        """

        if target.status == Constant.STATUS_EXIST:
            if not skip:
                if len(data.get('requests')) > 0:
                    status = 'OK' if data.get('success') else 'NG'
                else:
                    status = '-'
            else:
                status = 'SKIP'

            if args.group:
                group_status = 'FOUND' if data.get('group_found') else 'NEW'
            else:
                group_status = '-'
        else:
            status = target.status
            group_status = '-'

        data.set('status', status)
        data.set('group_status', group_status)

    def build(self, data, target):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        :rtype: object
        """

        if not data.header and target.identified:
            self.transform(data)

        data.map(self.filter_value)

        return self.build_row(data)

    def transform(self, data):
        """
        :param data:
        :type data: ValueData
        """

        data.update('ip_type', lambda v: 'PRIVATE' if v == Constant.IP_TYPE_PRIVATE else 'PUBLIC')

    def filter_value(self, v):
        return v

    @abstractmethod
    def build_row(self, data):  # pragma: no cover
        """
        :param data:
        :type data: ValueData
        :rtype: object
        """

        return None

    @abstractmethod
    def build_error(self, error):  # pragma: no cover
        return None

    def output_result(self, v):
        self.output(v)

    def output(self, v):
        print(v, flush=True)

    def output_complete(self, mode, args, rows):
        """
        :param mode:
        :type mode: int
        :param args:
        :type args: argparse.Namespace
        :param rows:
        :type rows: list
        """

        pass

    def transform_key_labels(self, data, mode):
        """
        :param data:
        :type data: ValueData
        :param mode:
        :type mode: int
        """

        pass

    def output_message(self, msg):
        pass

    def output_item(self, data):
        """
        :param data:
        :type data: ValueData
        """

        pass



import pytest

from ipsurv.configure.args_builder import ArgsBuilder
from ipsurv.core.pipeline import Pipeline
from ipsurv.configs import Config
from ipsurv.core.entity import Target, ValueData

import argparse
import re
import sys
import logging


class TestArgsBuilder:
    @pytest.fixture
    def args(self, mocker):
        args = mocker.Mock()
        args.format = None
        args.delimiter = ','
        args.fixed_delimiter = ','

        return args

    @pytest.fixture
    def args_builder(self, mocker):
        pipeline = Pipeline()

        args_builder = ArgsBuilder(Config, pipeline)

        return args_builder

    def test_parse(self, args, args_builder, monkeypatch):
        monkeypatch.setattr(sys, 'argv', ['ipsurv.py'])

        args = args_builder.parse()

        assert args.timeout == '8.0'

    def test_init_args(self, args, args_builder, capfd):
        arguments = {
            'verbose': {'default': 3, 'type': int, 'help': ''},
            'debug': {'default': False, 'help': '`--debug` is equivalent to `--verbose=3`.', 'action': 'store_true'},
            'log': {'default': 'abc.log', 'type': str, 'help': ''},
        }

        args_builder.init_args(arguments)

        captured = capfd.readouterr()

        assert re.search(r'Current:\s*3.+DEBUG', captured.out)

    def test_build_args(self, args, mocker, monkeypatch):
        monkeypatch.setattr(sys, 'argv', ['ipsurv.py', '--json=1', '--timeout=5'])

        pipeline = Pipeline()

        args_builder = ArgsBuilder(Config, pipeline)

        parent_parser = argparse.ArgumentParser(add_help=False, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

        parent_parser.parse_known_args()

        arguments = Config.ARGUMENTS

        env_args = {}
        env_conf = {}

        args = args_builder.build_args(parent_parser, arguments, env_args, env_conf)

        assert args.format == 'default'
        assert 'original' in args.fixed_format_params
        assert 'rdap' in args.fixed_collectors
        assert args.resolve is True

    def test_prepare_arguments(self, args_builder):
        parser = argparse.ArgumentParser(add_help=False, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

        args_builder._prepare_arguments(parser, Config.ARGUMENTS, {})

    def test_logging(self, args_builder, caplog):
        data = ValueData({})

        with caplog.at_level(logging.INFO):
            args_builder.logging(data, {}, {})

        print(caplog.text)

        assert re.search(r'ENV', caplog.text)

    def test_fix_validate_group(self, args):
        pipeline = Pipeline()

        args_builder = ArgsBuilder(Config, pipeline)

        assert args_builder._validate_group('network') == 'network'
        assert args_builder._validate_group('192.168.1.100') == '192.168.1.100'

        with pytest.raises(argparse.ArgumentTypeError):
            args_builder._validate_group('ABC')

    def test_fix_delimiter(self, args):
        pipeline = Pipeline()

        args_builder = ArgsBuilder(Config, pipeline)

        args.format = ''

        args.delimiter = ';'
        v = args_builder._fix_delimiter(args)
        assert v == ';'

        args.delimiter = None

        args.format = '1,2,3,4'
        v = args_builder._fix_delimiter(args)
        assert v == ','

        args.format = '1;2;3;4'
        v = args_builder._fix_delimiter(args)
        assert v == ';'

        args.format = '1\t2\t3\t4'
        v = args_builder._fix_delimiter(args)
        assert v == '\t'

    def test_fix_enclose(self, args):
        pipeline = Pipeline()

        args_builder = ArgsBuilder(Config, pipeline)

        args.enclose = '1'
        v = args_builder._fix_enclose(args)
        assert v == '"'

        args.enclose = '2'
        v = args_builder._fix_enclose(args)
        assert v == "'"

        args.enclose = '3'
        v = args_builder._fix_enclose(args)
        assert v == '|'

    def test_fix_collectors(self, args):
        pipeline = Pipeline()

        args_builder = ArgsBuilder(Config, pipeline)

        args.collect = 'rdap;dnstxt;ipinfo;dnsreverse;test'

        collectors = args_builder._fix_collectors(args)

        assert 'rdap' in collectors
        assert 'ipinfo' in collectors
        assert not ('test' in collectors)

    def test_notice(self, args, capfd):
        pipeline = Pipeline()

        args_builder = ArgsBuilder(Config, pipeline)

        args.json = True
        args.delimiter = True
        args.enclose = True

        args_builder._notice(args)

        captured = capfd.readouterr()

        assert re.search(r'option is ignored', captured.out)



import argparse
import json
import logging
import re
from collections import Counter

from ipsurv.configure.args_validators import FormatValidator, TimeoutValidator
from ipsurv.configs import Constant
from ipsurv.core.pipeline import Pipeline
from ipsurv.util.args_util import ArgsHelper, StdinLoader
from ipsurv.util.sys_util import System


class ArgsBuilder:
    def __init__(self, config, pipeline):
        self.config = config
        self.pipeline = pipeline  # type: Pipeline

    def parse(self):
        parent_parser, args = self.init_args(self.config.PRE_ARGUMENTS)

        if not args.disable_env:
            env_ags = StdinLoader.load_env(Constant.ENV_ARGS_VAR)
        else:
            env_ags = {}

        env_conf = self._load_env_conf(Constant.ENV_CONF_VAR)

        args = self.build_args(parent_parser, self.config.ARGUMENTS, env_ags, env_conf)

        return args

    def _load_env_conf(self, name):
        values = StdinLoader.load_env(name)

        env_conf = {}

        for k in self.config.ENV_CONFS:
            if k in values:
                env_conf[k] = values[k]

        return env_conf

    def init_args(self, arguments):
        parser, args = ArgsHelper.init_parser(arguments)

        if args.debug:
            args.verbose = 3

        ArgsHelper.init_logging(args.verbose, args.log)

        if args.verbose > 0:
            System.warn('Enable verbose mode. Current:' + str(args.verbose) + ' [Level - 1:TRACE_ERROR, 2:INFO, 3:DEBUG]')

            if args.log is not None:
                System.warn('Enable log.(File:' + args.log + ')')

        return parser, args

    def build_args(self, parent_parser, arguments, env_args, env_conf):
        desc = self._create_bottom_desc()

        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter, parents=[parent_parser], description=Constant.APP_DESCRIPTION, epilog=desc)

        self.pipeline.init_configure(arguments, env_args)

        self._prepare_arguments(parser, arguments, env_args)

        args = parser.parse_args()

        self._assign_shorten_option(args)

        if System.is_logging():
            self.logging(args, env_args, env_conf)

        self.pipeline.pre_configure(args, env_args, env_conf)

        args.conf = env_conf

        self._configure(parser, args)

        self.pipeline.post_configure(args, env_args, env_conf)

        self._notice(args)

        return args

    def _create_bottom_desc(self):
        desc = ''

        desc += "profiles in format:\n"
        desc += '  ' + ', '.join(self.config.FORMAT_PROFILES.keys()) + "\n\n"

        desc += Constant.APP_BOTTOM_DESC + "\n"

        return desc

    def _assign_shorten_option(self, args):
        if args.json_all:
            args.json = 2
            args.exhaustive = True

        if args.geoip_only:
            args.collect = 'geoip'
            args.format = 'area'

        if args.host_only:
            args.collect = 'dnsreverse'
            args.format = 'hostname'

    def _configure(self, parser, args):
        debug = True if System.get_log_level() == logging.DEBUG else False

        try:
            args.fixed_delimiter = self._fix_delimiter(args)
            args.fixed_format, args.fixed_format_params = self._fix_format(args, debug)
            args.fixed_timeout = self._fix_timeout(args, debug)
            args.fixed_enclose = self._fix_enclose(args)
            args.fixed_ranges = self._fix_ranges(args)
            args.fixed_collectors = self._fix_collectors(args)
        except Exception as e:
            logging.log(logging.DEBUG, 'Fix arguments error.', exc_info=True)

            parser.error(e)

    def _prepare_arguments(self, parser, arguments, env_args):
        parser.add_argument('target', type=str, default=None, nargs='*', help='IP addresses or FQDN or URL.')

        arguments['group']['type'] = self._validate_group

        ArgsHelper.add_arguments(parser, arguments, env_args)

    def logging(self, args, env_args, env_conf):
        System.output_data('ENV(' + Constant.ENV_ARGS_VAR + ')', env_args)

        System.output_data('ENV(' + Constant.ENV_CONF_VAR + ')', env_conf)

        params = vars(args)

        System.output_data('ARGUMENTS', params)

        v = json.dumps(params, ensure_ascii=False)
        System.output_body('ARGUMENTS_JSON', v, level=logging.INFO)

    def _validate_group(self, v):
        v = v.strip()

        if v == '':
            v = None
        elif not (v == 'network' or re.search(r'^([\d]{1,3}|[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3})$', v)):
            raise argparse.ArgumentTypeError("Illegal group value. Ex: network, 24, 255.255.255.0")

        return v

    def _fix_delimiter(self, args):
        v = args.delimiter

        if not v:
            matches = re.findall(r'[\t,;/|]', args.format)

            if len(matches) > 0:
                counter = Counter(matches)

                mc = counter.most_common(1)

                v, n = mc[0]
            else:
                v = Constant.DELIMITER_DEFAULT

        logging.log(logging.INFO, 'Fixed delimiter:' + v)

        return v

    def _fix_format(self, args, debug):
        format_validator = FormatValidator(self.config.FORMAT_PROFILES, self.config.FORMAT_PARAMS, debug=debug)

        return format_validator.validate(args)

    def _fix_timeout(self, args, debug):
        timeout_validator = TimeoutValidator(8.0, debug=debug)

        return timeout_validator.validate(args)

    def _fix_enclose(self, args):
        v = args.enclose

        if v == '1':
            v = '"'
        elif v == '2':
            v = "'"
        elif v == '3':
            v = '|'
        elif not v or v == '0':
            v = ''

        logging.log(logging.INFO, 'Fixed enclose:' + v)

        return v

    def _fix_ranges(self, args):
        ranges = re.split(r'[;, ]+', args.range)

        ranges = list(filter(lambda v: v.strip(), ranges))

        logging.log(logging.INFO, 'Fixed ranges:' + str(ranges))

        return ranges

    def _fix_collectors(self, args):
        v = args.collect.lower()

        collectors = re.split(r'[;, ]+', v)

        collectors = [v for v in collectors if v in self.config.COLLECTORS]

        logging.log(logging.INFO, 'Fixed collectors:' + str(collectors))

        return collectors

    def _notice(self, args):
        if args.json:
            if args.delimiter:
                System.warn('"delimiter" option is ignored in "json" mode.')

            if args.enclose:
                System.warn('"enclose" option is ignored in "json" mode.')
        else:
            if args.exhaustive:
                System.warn('You must use "exhaustive" option with "json" option.')

            if args.json_list:
                System.warn('You must use "json_list" option with "json" option.')

    def read_lines(self, args):
        if len(args.target) > 0:
            lines = args.target
        else:
            lines = StdinLoader.read_stdin(2.0)

        return lines



import logging

from ipsend.configs import Constant
from ipsend.service.sockets import Socket
from ipsurv import __version__
from ipsurv.util.sys_util import AppException
from ipsurv.util.sys_util import System


class ViewHelper:
    def show_head(self, args):
        pass

    def show_stopped(self):
        System.line(' Stopped by user...\n')

    def show_interactive_mode(self, ctrlkey):
        title = 'Line-break to send' if not ctrlkey else 'Ctrl-key to send'

        System.line('[INTERACTIVE] / ' + title + '\n')

        if ctrlkey:
            if not System.verify_os(True):
                msg = 'Press `Ctrl+D` to send.'
            else:
                msg = 'Press `Ctrl+Z` to send.'
        else:
            msg = 'Input a line break to send.'

        System.line('Please input send-data. ' + msg + '\n')

    def show_version(self):
        System.exit(Constant.APP_NAME + ' by ' + Constant.PYPI_NAME + ' ' + __version__)

    def get_hex_data(self, data):
        hex_data = ''.join(f'{byte:02x} ' for byte in data)

        return hex_data

    def output_error(self, e):
        msg = ''

        if not System.is_logging(logging.DEBUG):
            msg = '\n\nSet `--debug` or `--verbose=3` option to output error detail.'

        error = Socket.get_error(e)

        if error:
            System.warn(error + '\n')
        elif not isinstance(e, AppException):
            System.warn('An error has occurred.' + msg + '\n')
        else:
            System.warn(str(e) + msg + '\n')



import logging

from ipscap.configs import Constant
from ipscap.util.raw_socket_entity import IPHeader
from ipsurv import __version__
from ipsurv.util.sys_util import System
from ipsurv.util.sys_util import AppException
from datetime import datetime


class ViewHelper:
    TITLE_WIDTH = 120

    def show_head(self, args):
        System.line('Start capture packets...\n')

        if args.timeout is None:
            System.line('Press `Ctrl + C` to stop.\n')
        else:
            System.line("`--timeout` option is enabled. The capture will stop {} seconds automatically.".format(args.timeout) + "\n")

        if args.has_filters:
            self.show_filter_options(args)

    def show_filter_options(self, args):
        System.line('[FILTERS]')

        if args.protocol != 'TCP,UDP':
            self._show_option(args, 'protocol', 'fixed_protocols', lambda w: [IPHeader.get_protocol_code(v) for v in w])

        self._show_option(args, 'port', 'fixed_ports')

        self._show_option(args, 'ip', 'fixed_ips')

        if args.find:
            self._show_option(args, 'find')
            self._show_option(args, 'find_mode', 'fixed_find_mode')

        self._show_option(args, 'condition')

        self._show_option(args, 'tracking')

        System.line('')

    def _show_option(self, args, name, internal_name=None, fn=None):
        internal_name = internal_name if internal_name else name

        v = getattr(args, internal_name)

        if v:
            if fn:
                v = fn(v)

            System.line((name + ':').ljust(14) + str(v))

    def show_dumpfile_info(self, dumpfile):
        border = self.get_border()

        System.line(border + 'Captured Dump Logs'.center(self.TITLE_WIDTH) + border)
        System.line('Path:'.ljust(8) + dumpfile.get_path())
        System.line('Files:'.ljust(8) + str(dumpfile.get_file_num()))
        System.line('')

    def show_statistics(self, transfers, begin_tm, end_tm, args):
        self._show_stat_top(args.stat_mode)

        self._show_times(begin_tm, end_tm)

        self._show_stat_transfers(transfers, args.stat_group)

    def _show_stat_top(self, stat_mode):
        border = self.get_border()

        System.line(border + 'TRANSFER STATISTICS'.center(self.TITLE_WIDTH) + border)

        if stat_mode == 0:
            System.line('*The statistics is disabled by `--stat_mode` option.')
        elif stat_mode == 1:
            System.line('*The following is the statistics for captured transfers only. If you\'d like see to the statistics for all transfers, set`--stat_mode=2` option.')
        elif stat_mode == 2:
            System.line('*The following is the statistics for all transfers.')

        System.line("\n")

    def _show_times(self, begin_tm, end_tm):
        System.line('Begin time: '.ljust(16) + datetime.fromtimestamp(begin_tm).strftime('%Y-%m-%d %H:%M:%S'))
        System.line('End time: '.ljust(16) + datetime.fromtimestamp(end_tm).strftime('%Y-%m-%d %H:%M:%S'))

        running_sec = round(end_tm - begin_tm, 2)
        System.line('Running time: '.ljust(16) + str(running_sec) + 's')

        System.line("\n")

    def _show_stat_transfers(self, transfers, stat_group):
        if not stat_group:
            self._show_stat_transfer_items(transfers)
        else:
            self._show_stat_transfer_groups(transfers)

        System.line('')

    def _show_stat_transfer_items(self, transfers):
        for key, value in transfers.items():
            (protocol, src_ip, src_port, dest_ip, dest_port) = key

            protocol_code = IPHeader.get_protocol_code(protocol)

            System.line('[' + protocol_code + '] ' + src_ip + ':' + str(src_port) + ' <-> ' + dest_ip + ':' + str(dest_port))

            self._show_subtotal(IPHeader.DIRECTION_SEND, value)
            self._show_subtotal(IPHeader.DIRECTION_RECEIVE, value)

            System.line('')

    def _show_stat_transfer_groups(self, transfers):
        for key, value in transfers.items():
            (protocol, src_ip, dest_ip, port) = key

            protocol_code = IPHeader.get_protocol_code(protocol)

            protocol_ips = '[' + protocol_code + '] ' + src_ip + ' <-> ' + dest_ip
            System.line(protocol_ips.ljust(40) + ' Port: ' + str(port))

            self._show_subtotal(IPHeader.DIRECTION_SEND, value)
            self._show_subtotal(IPHeader.DIRECTION_RECEIVE, value)
            System.line(' GROUPS:'.ljust(12) + str(value['group_count']))

            System.line('')

    def show_stopped(self):
        System.line(' Stopped by user...\n')

    def _show_subtotal(self, direction, subtotals):
        subtotal = subtotals[direction]

        direction_code = IPHeader.get_direction_code(direction)

        line = 'count: ' + str(subtotal['count']) + ', ' + 'unique: ' + str(subtotal['unique']) + ', ' + 'size: ' + str(subtotal['size'])
        System.line((' ' + direction_code + ':').ljust(12) + line)

    def show_version(self):
        System.exit(Constant.APP_NAME + ' by ' + Constant.PYPI_NAME + ' ' + __version__)

    def show_nofilters(self):
        System.exit('Any filters are not specified. Set any filter option or`--force` option.', True)

    def output_debug(self, ip_header, protocol_header):
        if not System.is_logging():
            return

        line = ip_header.src_ip + ':' + str(protocol_header.src_port) + '(' + str(
            ip_header.src_ip_int) + ') -> ' + ip_header.dest_ip + ':' + str(
            protocol_header.dest_port) + '(' + str(ip_header.dest_ip_int) + ')'
        line += ', PACKET_LEN: ' + str(ip_header.total_length) + ', ' + ip_header.protocol_code
        line += ', DATA _LEN: ' + str(protocol_header.payload_length)

        logging.log(logging.INFO, line)

    def output_not_support(self, eth_header):
        if System.is_logging():
            hex_data = self.get_hex_data(eth_header)

            System.output_data('NOT_SUPPORT_PACKET', hex_data, level=logging.DEBUG)

    def get_hex_data(self, data):
        hex_data = ''.join(f'{byte:02x} ' for byte in data)

        return hex_data

    def output_error(self, e):
        msg = ''

        if not System.is_logging(logging.DEBUG):
            msg = '\n\nSet `--debug` or `--verbose=3` option to output error detail.'

        if not isinstance(e, AppException):
            System.warn('An error has occurred.' + msg + '\n')
        else:
            System.warn(str(e) + msg + '\n')

    def get_border(self, length=120):
        return "\n" + '*' * length + "\n"



import logging
import signal
import sys

from ipsend.configs import Config, Constant
from ipsend.core.object_factory import ObjectFactory
from ipsend.core.pipeline import Pipeline
from ipsend.service.packet_generator import PacketGenerator
from ipsend.service.view_helper import ViewHelper


class IpSendCmd:
    def __init__(self, factory):
        self.factory = factory  # type: ObjectFactory
        self.pipeline = self.factory.create_pipeline()  # type: Pipeline
        self.config = self.factory.get_config()  # type: Config

        self.socket = None
        self.data_input = None
        self.data_output = None

        self.view_helper = None  # type: ViewHelper

    def run(self):
        try:
            self._pre_initialize()

            args, parser = self._parse_args()

            self._initialize(args)

            self._verify_args(args, parser)

            self.dispatch(args)
        except Exception as e:
            self.view_helper.output_error(e)

            logging.log(logging.ERROR, str(e), exc_info=True)

    def _verify_args(self, args, parser):
        if args.version:
            self.view_helper.show_version()
        elif len(sys.argv) == 1:
            parser.print_help()
            sys.exit()

    def _pre_initialize(self):
        self.view_helper = self.factory.create_view_helper()

    def _initialize(self, args):
        self.data_input = self.factory.create_data_input()
        self.data_input.initialize(args.input)

        self.data_output = self.factory.create_data_output()
        self.data_output.initialize(args.output)

        self.socket = self._create_socket(args.mode, args.fixed_ssl_context)
        self.socket.initialize(self.data_input, self.data_output, args.mode, args.timeout)

        self.pipeline.initialize(self.config)

        signal.signal(signal.SIGINT, self.signal_stop)

    def _create_socket(self, mode, ssl_context):
        socket = None

        if mode in Constant.RICH_SOCKET_MODES:
            socket = self.factory.create_rich_socket()
            socket.set_ssl_context(ssl_context)
        elif mode in Constant.RAW_SOCKET_MODES:
            socket = self.factory.create_raw_socket()

        return socket

    def _parse_args(self):
        args_builder = self.factory.create_args_builder(self.config, self.pipeline)

        return args_builder.parse()

    def dispatch(self, args):
        if not args.interactive:
            self._run_instant(args)
        else:
            self._run_interactive(args)

        self._complete()

    def signal_stop(self, sig, frame):
        self.view_helper.show_stopped()

        self._complete()

        sys.exit()

    def _run_instant(self, args):
        data = self._create_send_data(args)

        if args.output_send > 0:
            byte_data = self.data_input.get_data(data)
            self.data_output.output_binary(byte_data)

            if args.output_send == 2:
                return

        self.socket.create(args.dest, args.port)

        self.socket.send(data)

        self.socket.receive()

        self.socket.close()

    def _create_send_data(self, args):
        if args.mode not in Constant.PAYLOAD_MODES:
            data = args.data

            if args.mode in Constant.RICH_SOCKET_MODES:
                if args.auto_lb and args.input == Constant.INPUT_TEXT:
                    data += '\n'
        else:
            packet_generator = PacketGenerator(self.factory, self.data_input)
            data = packet_generator.create_data(args)

        return data

    def _run_interactive(self, args):
        ctrlkey = True if args.interactive == 2 else False

        self.view_helper.show_interactive_mode(ctrlkey)

        socket_thread = self.factory.create_socket_thread(self.socket, self.view_helper)

        self.socket.create(args.dest, args.port)

        socket_thread.start()

        interactive_input = self.factory.create_interactive_input(ctrlkey)

        while True:
            line = interactive_input.get_input()

            if not self.socket.connected():
                print('Connection is closed unexpectedly.')
                break

            if line is not None:
                self.socket.send(line)

    def _complete(self):
        self.pipeline.complete()



import re

from ipscap.util.raw_socket_entity import IPHeader
from ipscap.configs import Constant
import codecs


class PacketFilter:
    def __init__(self, ev_parser):
        self._find_bytes = None
        self._trackings = []
        self._ev_parser = ev_parser

    def initialize(self, args):
        if args.find:
            self.prepare_find(args)

    def verify_capture(self, ip_header, protocol_header, args):
        is_capture = self.filter_packet(ip_header, protocol_header, args)

        if args.tracking:
            (ip1, ip2, port) = self._get_tacking_key(ip_header, protocol_header)

            if is_capture:
                tracking = (ip_header.src_ip, ip_header.dest_ip, port)

                if tracking not in self._trackings:
                    self._add_transfer(ip1, ip2, port)
            else:
                if self._is_tracking_transfer(ip1, ip2, port):
                    is_capture = True

        return is_capture

    def _get_tacking_key(self, ip_header, protocol_header):
        port = protocol_header.dest_port if protocol_header.dest_port < protocol_header.src_port else protocol_header.src_port

        if ip_header.src_ip > ip_header.dest_ip:
            return (ip_header.dest_ip, ip_header.src_ip, port)

        return (ip_header.src_ip, ip_header.dest_ip, port)

    def _add_transfer(self, ip1, ip2, port):
        tracking = (ip1, ip2, port)

        self._trackings.append(tracking)

    def _is_tracking_transfer(self, ip1, ip2, port):
        is_tracking = False

        for tracking in self._trackings:
            if tracking[0] == ip1 and tracking[1] == ip2 and tracking[2] == port:
                is_tracking = True
                break

        return is_tracking

    def filter_packet(self, ip_header, protocol_header, args):
        if not self.verify_protocol(ip_header, args):
            return False

        if not self.verify_ip(ip_header, args):
            return False

        if not self.verify_port(protocol_header, args):
            return False

        if not self.verify_find(ip_header, protocol_header, args):
            return False

        if not self._ev_parser.is_empty():
            if not self.verify_condition(ip_header, protocol_header):
                return False

        return True

    def verify_protocol(self, ip_header, args):
        if IPHeader.PROTOCOL_TCP in args.fixed_protocols and ip_header.protocol == IPHeader.PROTOCOL_TCP:
            return True

        if IPHeader.PROTOCOL_UDP in args.fixed_protocols and ip_header.protocol == IPHeader.PROTOCOL_UDP:
            return True

        if IPHeader.PROTOCOL_ICMP in args.fixed_protocols and ip_header.protocol == IPHeader.PROTOCOL_ICMP:
            return True

        return False

    def verify_ip(self, ip_header, args):
        if args.fixed_ips is not None:
            for ip in args.fixed_ips:
                if ip_header.src_ip == ip or ip_header.dest_ip == ip:
                    return True

            return False

        return True

    def verify_port(self, protocol_header, args):
        if args.fixed_ports is not None:
            for port in args.fixed_ports:
                if protocol_header.src_port == port or protocol_header.dest_port == port:
                    return True

            return False

        return True

    def prepare_find(self, args):
        find_mode = args.find_mode

        if find_mode == Constant.FIND_REGEX or find_mode == Constant.FIND_MATCH:
            self._find_bytes = args.find.encode('utf-8')
        elif find_mode == Constant.FIND_BINARY:
            self._find_bytes = self.create_bytes_by_binary(args.find)
        elif find_mode == Constant.FIND_HEX:
            self._find_bytes = self.create_bytes_by_hex(args.find)

    def verify_find(self, ip_header, protocol_header, args):
        find_mode = args.find_mode

        if args.find:
            if find_mode == Constant.FIND_REGEX or find_mode == Constant.FIND_MATCH:
                flags = re.IGNORECASE if find_mode == Constant.FIND_REGEX else 0

                if not re.search(self._find_bytes, protocol_header.payload_data, flags):
                    return False
            elif find_mode == Constant.FIND_BINARY:
                transfer_data = ip_header.header_data + protocol_header.header_data + protocol_header.payload_data

                if self._find_bytes not in transfer_data:
                    return False
            elif find_mode == Constant.FIND_HEX:
                transfer_data = ip_header.header_data + protocol_header.header_data + protocol_header.payload_data

                if self._find_bytes not in transfer_data:
                    return False

        return True

    def verify_condition(self, ip_header, protocol_header):
        if self._ev_parser.assigned('port'):
            port = protocol_header.dest_port if protocol_header.dest_port < protocol_header.src_port else protocol_header.src_port

            if not self._ev_parser.evaluate('port', port):
                return False

        if self._ev_parser.assigned('client_port'):
            port = protocol_header.dest_port if protocol_header.dest_port > protocol_header.src_port else protocol_header.src_port

            if not self._ev_parser.evaluate('client_port', port):
                return False

        if self._ev_parser.assigned('src_port'):
            if not self._ev_parser.evaluate('src_port', protocol_header.src_port):
                return False

        if self._ev_parser.assigned('dest_port'):
            if not self._ev_parser.evaluate('dest_port', protocol_header.dest_port):
                return False

        if self._ev_parser.assigned('ttl'):
            if not self._ev_parser.evaluate('ttl', ip_header.ttl):
                return False

        if ip_header.protocol == IPHeader.PROTOCOL_TCP:
            if self._ev_parser.assigned('flags'):
                flags = self._ev_parser.get_value('flags')

                if not (protocol_header.flags & flags):
                    return False

            if self._ev_parser.assigned('seq'):
                if not self._ev_parser.evaluate('seq', protocol_header.seq_no):
                    return False

            if self._ev_parser.assigned('ack'):
                if not self._ev_parser.evaluate('ack', protocol_header.ack_no):
                    return False

            if self._ev_parser.assigned('window'):
                if not self._ev_parser.evaluate('window', protocol_header.window):
                    return False

            if self._ev_parser.assigned('mss'):
                if not self._ev_parser.evaluate('mss', protocol_header.tcp_options.get('mss')):
                    return False

            if self._ev_parser.assigned('wscale'):
                if not self._ev_parser.evaluate('wscale', protocol_header.tcp_options.get('wscale')):
                    return False

            if self._ev_parser.assigned('sack'):
                if not self._ev_parser.evaluate('sack', protocol_header.tcp_options.get('sack')):
                    return False

        return True

    def create_bytes_by_binary(self, data):
        return codecs.decode(data, 'unicode_escape').encode('latin1')

    def create_bytes_by_hex(self, data):
        data = re.sub(r'\s', '', data)

        try:
            if len(data) % 2 != 0:
                raise Exception()

            byte_data = bytes.fromhex(data)
        except Exception:
            raise Exception('Hex data parse error.')

        return byte_data



import re
import socket
import ssl
import urllib.parse
import urllib.request
import urllib.error
from collections import OrderedDict

from ipsurv.requester.requester import Requester


class HttpRequester(Requester):
    """
    :param timeout:
    :type timeout: float

    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#requester
    """

    def __init__(self, timeout=None):
        super().__init__(timeout)

        self.host = None

        self.headers = {
            'User-Agent': 'IpSurv requester',
            'Accept-Language': 'en-US,en;q=0.5'
        }

    def set_headers(self, headers):
        self.headers = headers

    def request(self, url, default_encoding='utf-8'):
        res, body = self.request_http(url)

        success = False
        response = OrderedDict()

        if res.status != 0:
            response['http_status'] = res.status
            response['http_size'] = len(body)
            response['http_server'] = res.getheader('Server')

            mime, encoding = self._parse_content_type(res, default_encoding)
            response['http_mime'] = mime
            response['headers'] = res.getheaders()
            response['body'] = self._get_body(body, encoding)
            response['headers'] = res.getheaders()

            success = True
        else:
            raise self._http_exception(res, body)

        return success, response

    def _parse_content_type(self, res, default_encoding):
        content_type = res.getheader('Content-Type')

        params = content_type.split('charset=')

        if len(params) == 1:
            encoding = default_encoding
        else:
            encoding = params[1]

        mime = params[0].strip('; ')

        return mime, encoding

    def _get_body(self, body, encoding):
        try:
            body.decode(encoding)
        except Exception:
            pass

        return body

    def request_http(self, url):
        url = self._create_url(url)

        req = urllib.request.Request(url)

        for name, value in self.headers.items():
            req.add_header(name, value)

        try:
            with urllib.request.urlopen(req, timeout=self.timeout) as res:
                body = res.read()
        except urllib.error.URLError as e:
            res = e
            body = res.read()

        return res, body

    def request_alpn_h2(self, url, port=443):
        url = self._create_url(url)

        parsed_url = urllib.parse.urlparse(url)

        host = parsed_url.netloc

        context = ssl.create_default_context()

        context.set_alpn_protocols(['h2'])

        try:
            with socket.create_connection((host, port), timeout=self.timeout) as sock:
                with context.wrap_socket(sock, server_hostname=host) as ssock:
                    negotiated_protocol = ssock.selected_alpn_protocol()
                    if negotiated_protocol == 'h2':
                        return 1
        except Exception:
            return -1

        return 0

    def _create_url(self, url):
        if not re.search(r'^https?:\/\/', url, flags=re.IGNORECASE):
            url = 'http://' + url
        elif re.search(r'^\/\/', url, flags=re.IGNORECASE):
            url = 'http:' + url

        return url



import argparse
import logging
import ssl

from ipsend.configs import Constant
from ipsend.core.pipeline import Pipeline
from ipsurv.util.args_util import ArgsHelper
from ipsurv.util.sys_util import System


class ArgsBuilder:
    def __init__(self, config, pipeline):
        self.config = config
        self.pipeline = pipeline  # type: Pipeline

    def parse(self):
        parent_parser, args = self.init_args(self.config.PRE_ARGUMENTS)

        return self.build_args(parent_parser, self.config.ARGUMENTS)

    def init_args(self, arguments):
        parser, args = ArgsHelper.init_parser(arguments)

        if args.debug:
            args.verbose = 3

        ArgsHelper.init_logging(args.verbose, args.log)

        if args.verbose > 0:
            System.warn('Enable verbose mode. Current:' + str(args.verbose) + ' [Level - 1:TRACE_ERROR, 2:INFO, 3:DEBUG]')

            if args.log is not None:
                System.warn('Enable log.(File:' + args.log + ')')

        return parser, args

    def build_args(self, parent_parser, arguments):
        desc = self._create_bottom_desc()

        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter, parents=[parent_parser], description=Constant.APP_DESCRIPTION, epilog=desc)

        self.pipeline.init_configure(arguments)

        self._prepare_arguments(parser, arguments)

        args = parser.parse_args()

        self._assign_shorten_option(args)

        if System.is_logging():
            self.logging(args)

        self._configure(parser, args)

        self._validate_options(parser, args, arguments)

        self._notice(args)

        return (args, parser)

    def _create_bottom_desc(self):
        desc = ''

        desc += Constant.APP_BOTTOM_DESC + "\n"

        return desc

    def _assign_shorten_option(self, args):
        if args.http:
            args.port = 80

        if args.https:
            args.port = 443
            args.mode = Constant.MODE_SSL

        if args.I:
            args.interactive = 1

    def _configure(self, parser, args):
        try:
            args.fixed_ssl_context = self._fix_ssl_context(args)
            self._fix_dest(args)
        except Exception as e:
            logging.log(logging.DEBUG, 'Fix arguments error.', exc_info=True)

            parser.error(e)

    def _fix_ssl_context(self, args):
        if not args.ssl_context:
            ssl_context = None
        else:
            k = args.ssl_context.lower()

            if k in Constant.SSL_CONTEXTS:
                ssl_context = getattr(ssl, Constant.SSL_CONTEXTS[k], None)

                if ssl_context is None:
                    raise Exception('Not support SSL context.')
            else:
                raise Exception('Unknown SSL context.')

        return ssl_context

    def _fix_dest(self, args):
        if args.mode in Constant.RAW_SOCKET_MODES or args.mode in Constant.PAYLOAD_MODES:
            if not args.dest_ip and args.dest:
                args.dest_ip = args.dest

            if not args.dest_port and args.port:
                args.dest_port = args.port

    def _validate_options(self, parser, args, arguments):
        _, rawargs = ArgsHelper.init_parser(arguments, raw=True)

        mode = args.mode

        if mode in Constant.RICH_SOCKET_MODES:
            self._validate_options_by_mode(mode, parser, rawargs, Constant.RAW_SOCKET_OPTIONS)

            if not args.dest:
                parser.error('`--dest` is required in `{}` mode.'.format(mode))

            if args.port <= 0:
                parser.error('`--port` is required in `{}` mode.'.format(mode))
        elif mode in Constant.RAW_SOCKET_MODES or mode in Constant.PAYLOAD_MODES:
            self._validate_options_by_mode(mode, parser, rawargs, Constant.RICH_SOCKET_OPTIONS)

        if mode != Constant.MODE_SSL:
            self._validate_options_by_mode(mode, parser, rawargs, Constant.SSL_OPTIONS)

        if args.interactive:
            if args.data:
                parser.error('`data` can not be specified with `--interactive` option.')

            if args.output_send:
                parser.error('`output_send` can not be specified specified with `--interactive` option.')

    def _validate_options_by_mode(self, mode, parser, args, non_options):
        non_supports = []

        for key in non_options:
            if getattr(args, key):
                non_supports.append(key)

        if len(non_supports) > 0:
            msg = ', '.join(['`--' + key + '`' for key in non_supports])
            msg += ' option is not supported in `{}` mode.'.format(mode)
            parser.error(msg)

    def _prepare_arguments(self, parser, arguments):
        ArgsHelper.add_arguments(parser, arguments, {}, group_names=self.config.ARGUMENTS_GROUP_NAMES)

    def logging(self, args):
        params = vars(args)

        System.output_data('ARGUMENTS', params)

    def _notice(self, args):
        pass



import pytest

from ipsurv.core.entity import Target, HeaderTarget, TargetGroup
from ipsurv.core.entity import ValueData, ValueDataFactory
import re


class TestTarget:
    def test(self, mocker):
        target = Target('test_target')
        target.identifier = 'identifier'

        assert target.raw == 'test_target'
        assert str(target) == 'identifier'


class TestHeaderTarget:
    def test(self, mocker):
        target = HeaderTarget('test_target')
        target.identifier = 'identifier'

        assert target.xyz == 'xyz'
        assert str(target) == 'target'

    def test_dump(self, mocker, capfd):
        target = HeaderTarget('test_target')
        target.identifier = 'identifier'

        target.dump()

        captured = capfd.readouterr()

        assert re.search(r'identifier', captured.out)


class TestTargetGroup:
    def test(self, mocker):
        target = TargetGroup(1, 2, 'abc')

        assert target.begin_int == 1
        assert target.value == 'abc'

        assert target.get_values()['begin_int'] == 1

    def test_dump(self, mocker, capfd):
        target = TargetGroup(1, 2, 'abc')
        target.dump()

        captured = capfd.readouterr()

        assert re.search(r'begin_int', captured.out)


class TestValueData:
    def test(self, mocker):
        data = ValueData({'a': 1, 'b': 2, 'c': 3, 'e': 99})

        data.update('c', lambda v: v + 1)
        data.set('d', 4)
        data.delete('e')

        assert data.get('a') == 1
        assert data.get('b') == 2
        assert data.get('c') == 4
        assert data.get('d') == 4
        assert data.get('e') is None

        data.set_header(True)

        assert data.header is True

    def test_map(self, mocker):
        data = ValueData({'a': 1, 'b': 2, 'c': 3, 'e': 99})

        data.map(lambda v: v + 1)

        assert data.get('a') == 2

    def test_data(self, mocker):
        data = ValueData({})

        data.set_data({'a': 1, 'b': 2, 'c': 3, 'e': 99})

        assert data.get('a') == 1
        assert data.get('b') == 2
        assert data.get('c') == 3

    def test_get_values(self, mocker):
        data = ValueData({'a': 1, 'b': 2, 'c': 3})

        values = data.get_values()

        assert values['a'] == 1

    def test_dump(self, mocker, capfd):
        data = ValueData({'a': 1234})
        data.dump()

        captured = capfd.readouterr()

        assert re.search(r'1234', captured.out)


class TestValueDataFactory:
    def test(self, mocker):
        factory = ValueDataFactory({'a': 1, 'b': 2, 'c': 3, 'd': 5}, ['a', 'b', 'c', 'e'])

        data = factory.create()

        assert data.get('a') == 1
        assert data.get('b') == 2
        assert data.get('c') == 3
        assert data.get('e') is None

    def test_build(self, mocker):
        factory = ValueDataFactory({'a': 1, 'b': 2, 'c': 3, 'd': 5}, ['a', 'b', 'c', 'e'])

        assert type(factory.build({})) == ValueData



import pytest

from ipsurv.configure.args_validators import FormatValidator, TimeoutValidator
import re


class TestFormatValidator:

    @pytest.fixture
    def args(self, mocker):
        args = mocker.Mock()
        args.format = None
        args.delimiter = ','
        args.fixed_delimiter = ','
        args.sequence = True
        args.add_ip = False

        return args

    def test_validator(self, args):
        profiles = {
            'ip': ['ip', 'ip2']
        }

        params = ['success', 'sequence', 'ip', 'ip2', 'identifier', 'test']

        validator = FormatValidator(profiles, params)

        args.format = '{ip}'
        args.ident = True

        format, params = validator.validate(args)

        assert format == '{sequence},{identifier},{ip}'
        assert 'ip' in params

        args.format = '<ip>'

        format, params = validator.validate(args)

        assert format == '{sequence},{identifier},{ip},{ip2}'
        assert 'ip2' in params

        args.format = '<ip>,{test}'

        format, params = validator.validate(args)

        assert format == '{sequence},{identifier},{ip},{ip2},{test}'
        assert 'test' in params

        format, params = validator._append_head(args, '{ip}', params)

        assert 'sequence' in params

        format, params = validator._create_by_profile(args, 'ip')

        assert 'ip' in params

    def test_create_by_params(self, args):
        profiles = {
            'ip': ['ip', 'ip2']
        }

        params = ['success', 'sequence', 'ip', 'ip2', 'identifier', 'test']

        validator = FormatValidator(profiles, params)

        format = validator._create_by_params(params, args)

        assert re.search(r'ip', format)

    def test_test_format(self, args):
        validator = FormatValidator(None, None)

        assert validator._test_format('{ip}', {'ip': 123}) is None

        with pytest.raises(Exception):
            validator._test_format('{ip2}', {'ip': 123})

    def test_misc(self, args):
        profiles = {
            'ip': ['ip', 'ip2']
        }

        params = ['success', 'sequence', 'ip', 'ip2', 'identifier', 'test']

        validator = FormatValidator(profiles, params)

        args.format = '{test1},<test2>'

        with pytest.raises(Exception):
            validator.validate(args)


class TestTimeoutValidator:
    @pytest.fixture
    def args(self, mocker):
        args = mocker.Mock()
        args.timeout = None

        return args

    def test_validator(self, args):
        validator = TimeoutValidator()

        args.timeout = '8.1'

        timeout = validator.validate(args)

        assert timeout['dns'] == 8.1
        assert timeout['http'] == 8.1
        assert timeout['reactivity'] == 8.1

        args.timeout = '8;5;3'

        timeout = validator.validate(args)

        assert timeout['dns'] == 8
        assert timeout['http'] == 5
        assert timeout['reactivity'] == 3

        args.timeout = '1,5,2'

        timeout = validator.validate(args)

        assert timeout['dns'] == 1
        assert timeout['http'] == 5
        assert timeout['reactivity'] == 2

        args.timeout = '8;3'

        timeout = validator.validate(args)

        assert timeout['dns'] == 8
        assert timeout['http'] == 3
        assert timeout['reactivity'] == 8



from ipscap.util.raw_socket_entity import IPHeader


class TransferStore:
    def __init__(self):
        self.transfers = {}

    def add(self, ip_header, protocol_header):
        key = self._get_transfer_key(ip_header, protocol_header)

        if key not in self.transfers:
            self.transfers[key] = {IPHeader.DIRECTION_SEND: {'count': 0, 'unique': 0, 'size': 0},
                                   IPHeader.DIRECTION_RECEIVE: {'count': 0, 'unique': 0, 'size': 0},
                                   'cur': 0}

        transfer = self.transfers[key][ip_header.direction]
        transfer['count'] += 1
        transfer['size'] += protocol_header.payload_length

        passage_num = transfer['count']

        if len(protocol_header.get_sanitized_data()) > 0:
            cur = self.transfers[key]['cur']

            if cur != ip_header.direction:
                transfer['unique'] += 1
                self.transfers[key]['cur'] = ip_header.direction

        return (passage_num)

    def _get_transfer_key(self, ip_header, protocol_header):
        key = None

        if ip_header.direction == IPHeader.DIRECTION_SEND:
            key = (ip_header.protocol, ip_header.src_ip, protocol_header.src_port, ip_header.dest_ip, protocol_header.dest_port)
        elif ip_header.direction == IPHeader.DIRECTION_RECEIVE:
            key = (ip_header.protocol, ip_header.dest_ip, protocol_header.dest_port, ip_header.src_ip, protocol_header.src_port)

        return key

    def totalize(self, stat_group):
        sorted_transfers = dict(sorted(self.transfers.items()))

        if stat_group:
            transfers = self.totalize_groups(sorted_transfers, stat_group)
        else:
            transfers = sorted_transfers

        return transfers

    def totalize_groups(self, sorted_transfers, stat_group):
        transfers = dict()

        for key, transfer in sorted_transfers.items():
            (protocol, first_ip, first_port, second_ip, second_port) = key

            port = first_port if first_port < second_port else second_port

            if stat_group == 1:
                key = (protocol, first_ip, second_ip, port)
            else:
                key = (protocol, first_ip, second_ip, -1)

            if key not in transfers:
                transfers[key] = {IPHeader.DIRECTION_SEND: {'count': 0, 'unique': 0, 'size': 0},
                                  IPHeader.DIRECTION_RECEIVE: {'count': 0, 'unique': 0, 'size': 0},
                                  'group_count': 0}

            rtransfer = transfers[key]
            rtransfer['group_count'] += 1

            self.totalize_direction(rtransfer, transfer, IPHeader.DIRECTION_SEND)
            self.totalize_direction(rtransfer, transfer, IPHeader.DIRECTION_RECEIVE)

            transfers[key] = rtransfer

        return transfers

    def totalize_direction(self, rtransfer, transfer, direction):
        rtrans = rtransfer[direction]
        trans = transfer[direction]

        for k, v in trans.items():
            rtrans[k] += v



import socket
import struct
from abc import ABC

from ipscap.util.raw_socket_entity import TCPHeader, UDPHeader


class IPHeaderGenerator:
    def __init__(self):
        pass

    def generate(self, ip_header):
        version_ihl = (ip_header.version << 4) + ip_header.iph_length

        flags_fragment_offset = (ip_header.flags << 13) + ip_header.fragment_offset

        src_ip = socket.inet_aton(ip_header.src_ip)
        dest_ip = socket.inet_aton(ip_header.dest_ip)

        payload_length = len(ip_header.payload_data)

        ip_header.total_length = (ip_header.iph_length * 4) + payload_length

        header = [
            version_ihl,
            ip_header.tos,
            ip_header.total_length,
            ip_header.identification,
            flags_fragment_offset,
            ip_header.ttl,
            ip_header.protocol,
            0,
            src_ip,
            dest_ip
        ]

        byte_data = struct.pack('!BBHHHBBH4s4s', *header)

        checksum = self.calculate_checksum(byte_data)

        header[7] = checksum
        ip_header_data = struct.pack('!BBHHHBBH4s4s', *header)

        byte_data = ip_header_data + ip_header.payload_data

        return byte_data

    def calculate_checksum(self, header):
        if len(header) % 2 == 1:
            header += b'\x00'

        checksum = 0
        for i in range(0, len(header), 2):
            part = (header[i] << 8) + header[i + 1]
            checksum += part
            checksum = (checksum & 0xffff) + (checksum >> 16)

        checksum = ~checksum & 0xffff

        return checksum


class HeaderGenerator(ABC):
    def generate(self, protocol_header, src_ip=None, dest_ip=None):
        pass

    def create_pseudo_header(self, src_ip, dest_ip, protocol, total_length):
        src_ip = socket.inet_aton(src_ip)
        dest_ip = socket.inet_aton(dest_ip)
        reserved = 0

        pseudo_header = struct.pack('!4s4sBBH', src_ip, dest_ip, reserved, protocol, total_length)

        return pseudo_header

    def calculate_checksum(self, data):
        if len(data) % 2 == 1:
            data += b'\x00'

        checksum = 0
        for i in range(0, len(data), 2):
            part = (data[i] << 8) + data[i + 1]
            checksum += part
            checksum = (checksum & 0xffff) + (checksum >> 16)

        checksum = ~checksum & 0xffff
        return checksum


class ICMPHeaderGenerator(HeaderGenerator):
    def generate(self, icmp_header, src_ip=None, dest_ip=None):
        header = [
            icmp_header.type,
            icmp_header.code,
            0,
            icmp_header.identifier,
            icmp_header.sequence
        ]

        byte_data = struct.pack('!BBHHH', *header)

        checksum = self.calculate_checksum(byte_data + icmp_header.payload_data)

        header[2] = checksum
        icmp_header_data = struct.pack('!BBHHH', *header)

        byte_data = icmp_header_data + icmp_header.payload_data

        return byte_data


class TCPHeaderGenerator(HeaderGenerator):
    def generate(self, tcp_header, src_ip=None, dest_ip=None):
        if tcp_header.tcph_length <= 0:
            tcp_header.tcph_length = TCPHeader.DEFAULT_HEADER_LEN

        data_offset = tcp_header.tcph_length // 4
        offset_flags = (data_offset << 12) + (tcp_header.flags & 0x1FF)

        header = [
            tcp_header.src_port,
            tcp_header.dest_port,
            tcp_header.seq_no,
            tcp_header.ack_no,
            offset_flags,
            tcp_header.window,
            0,
            tcp_header.urgent_pointer
        ]

        byte_data = struct.pack('!HHLLHHHH', *header)

        total_length = len(byte_data) + len(tcp_header.payload_data)

        pseudo_header = self.create_pseudo_header(src_ip, dest_ip, socket.IPPROTO_TCP, total_length)

        checksum = self.calculate_checksum(pseudo_header + byte_data + tcp_header.payload_data)

        header[6] = checksum
        tcp_header_data = struct.pack('!HHLLHHHH', *header)

        byte_data = tcp_header_data + tcp_header.payload_data

        return byte_data


class UDPHeaderGenerator(HeaderGenerator):
    def create_udp_header(self):
        return UDPHeader()

    def generate(self, udp_header, src_ip=None, dest_ip=None):
        if udp_header.udph_length <= 0:
            udp_header.udph_length = UDPHeader.DEFAULT_HEADER_LEN + len(udp_header.payload_data)

        header = [
            udp_header.src_port,
            udp_header.dest_port,
            udp_header.udph_length,
            0
        ]

        byte_data = struct.pack('!HHHH', *header)

        pseudo_header = self.create_pseudo_header(src_ip, dest_ip, socket.IPPROTO_UDP, udp_header.udph_length)

        checksum = self.calculate_checksum(pseudo_header + byte_data + udp_header.payload_data)

        header[3] = checksum
        udp_header_data = struct.pack('!HHHH', *header)

        byte_data = udp_header_data + udp_header.payload_data

        return byte_data



import pytest
from unittest.mock import patch
from ipsurv.core.object_factory import ObjectFactory
from ipsurv.ip_surv_cmd import IpSurvCmd
from ipsurv.data_collector.self_collector import SelfCollector
from ipsurv.util.args_util import StdinLoader
import re
import sys
import io
import os
import logging


class TestIpSurvCmd:
    @pytest.fixture
    def ip_surv_cmd(self, mocker):
        factory = ObjectFactory()

        ip_surv_cmd = IpSurvCmd(factory)

        return ip_surv_cmd

    def test_ip_surv_cmd(self, capsys, monkeypatch, ip_surv_cmd):
        monkeypatch.setattr(sys, 'argv', ['ipsurv.py'])

        with patch('ipsurv.util.args_util.StdinLoader.read_stdin', return_value=[]):
            ip_surv_cmd.run()

            captured = capsys.readouterr()

            assert re.search(r'No target data', captured.out)

    def test_version(self, capsys, monkeypatch, ip_surv_cmd):
        monkeypatch.setattr(sys, 'argv', ['ipsurv.py', '--version'])
        monkeypatch.setattr(os, '_exit', lambda v: 0)

        with patch('ipsurv.util.args_util.StdinLoader.read_stdin', return_value=[]):
            ip_surv_cmd.run()

            captured = capsys.readouterr()

            assert re.search(r'ipsurv', captured.out)

    def test_options(self, capsys, monkeypatch, ip_surv_cmd, caplog):
        monkeypatch.setattr(sys, 'argv', ['ipsurv.py', '--headers=2', '--begin=2', '--verbose=3', '--group=8', '--icmp=1', '--skip_duplicate=1'])
        monkeypatch.setattr(os, '_exit', lambda v: 0)

        with patch('ipsurv.util.args_util.StdinLoader.read_stdin', return_value=['192.168.1.100', '192.168.1.101', '192.168.1.102', 'sample']):
            with caplog.at_level(logging.INFO):
                ip_surv_cmd.run()

                captured = capsys.readouterr()

                assert re.search(r'ILLEGAL_FORMAT', captured.out)

    def test_ips(self, capsys, monkeypatch, ip_surv_cmd):
        monkeypatch.setattr(sys, 'argv', ['ipsurv.py', '192.168.1.100'])
        monkeypatch.setattr(os, '_exit', lambda v: 0)

        with patch('ipsurv.util.args_util.StdinLoader.read_stdin', return_value=[]):
            ip_surv_cmd.run()

            captured = capsys.readouterr()

            assert re.search(r'192.168.1.100', captured.out)

    def test_self(self, capsys, monkeypatch, ip_surv_cmd):
        monkeypatch.setattr(sys, 'argv', ['ipsurv.py', 'self'])
        monkeypatch.setattr(os, '_exit', lambda v: 0)
        monkeypatch.setattr(SelfCollector, 'request_data', lambda target, requires: True, {'ip': None})

        with patch('ipsurv.util.args_util.StdinLoader.read_stdin', return_value=[]):
            ip_surv_cmd.run()

            captured = capsys.readouterr()

            assert re.search(r'Data not found', captured.out)



import json
import logging
import re

from ipsurv.requester.requester import Requester


class IpInfoRequester(Requester):
    """
    :param timeout:
    :type timeout: float
    :param token:
    :type token: str

    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#requester
    """

    def __init__(self, timeout=None, token=None):
        super().__init__(timeout)

        self.host = 'ipinfo.io'
        self.token = token

        self.headers = {
            'User-Agent': 'Requester',
            'Accept-Language': 'en-US,en;q=0.5'
        }

    def set_headers(self, headers):
        self.headers = headers

    def request(self, ip):
        res, body = self.request_ip(ip)

        success = False
        response = {}

        if res.status == 200:
            response = json.loads(body)

            response = self._fill_data(response)

            success = True
        else:
            raise self._http_exception(res, body)

        return success, response

    def _fill_data(self, response):
        if response.get('org') is not None:
            match = re.search(r'^(AS\d+)\s(.+)$', response.get('org'))
            response['asn'] = match.group(1)
            response['org'] = match.group(2)

        return response

    def request_ip(self, ip):
        if not ip:
            path = '/json?'
        else:
            path = '/' + ip + '/json?'

        if self.token:
            path += 'token=' + self.token

        url = 'https://' + self.host + path
        logging.info('IPINFO_URL:' + url)

        conn = self._create_http_connection(self.host)

        try:
            conn.request('GET', path, headers=self.headers)

            res = conn.getresponse()

            body = res.read()
        except Exception as e:
            raise e
        finally:
            conn.close()

        return res, body



import io
import re
import tokenize

'''
ev_parser = EvaluationParser({
    'a': {'type': int},
    'b': {'type': int, 'single': True},
    'c': {'type': int},
    'd': {'type': lambda v: v.upper()},
    'e': {'type': float},
    'f': {'type': lambda v: v.upper(), 'types': method, 'list': True, 'single': True},
})

value = 'a>= 1; a<6 ; b=8; c=80,443; d=a,b; e=4.5'

ev_parser.parse(value)
'''


class EvaluationParser:
    OPERATOR_EQUAL = 1
    OPERATOR_NOT_EQUAL = 2
    OPERATOR_GREATER = 4
    OPERATOR_LESS = 8

    def __init__(self, rules=None):
        self.rules = rules
        self.items = {}

    def initialize(self, rules):
        self.rules = rules

    def parse(self, rule_value):
        if self.rules is None:
            raise Exception('Rules don\'t exist.')

        self.items = {}

        readline = io.StringIO(rule_value).readline
        tokens = tokenize.generate_tokens(readline)

        cur_var = None
        cur_op = 0
        cur_values = []

        for tok in tokens:
            if tok.type == tokenize.NAME:
                if cur_var is None:
                    cur_var = tok.string
                else:
                    cur_values.append(tok.string)
            elif tok.type == tokenize.OP:
                if re.search(r'^[!=<>]+$', tok.string):
                    cur_op = self._add_operators(cur_op, tok.string)
                elif tok.string == ';':
                    self._append_item(cur_var, cur_op, cur_values)
                    cur_var = None
                    cur_op = 0
                    cur_values = []
                elif tok.string == '-':
                    next_token = next(tokens)

                    if next_token.type == tokenize.NUMBER:
                        cur_values.append('-' + next_token.string)
                    else:
                        self._raise_parse_error(cur_var)
            elif tok.type == tokenize.NUMBER:
                cur_values.append(tok.string)
            elif tok.type == tokenize.NEWLINE or tok.type == tokenize.ENDMARKER:
                continue

        if cur_var and cur_op:
            self._append_item(cur_var, cur_op, cur_values)
            cur_var = None
            cur_op = 0

        if cur_var or cur_op:
            self._raise_parse_error(rule_value)

        return self.items

    def _add_operators(self, cur_op, value):
        if re.search(r'!=', value):
            cur_op |= self.OPERATOR_NOT_EQUAL
        elif re.search(r'=', value):
            cur_op |= self.OPERATOR_EQUAL

        if re.search(r'>', value):
            cur_op |= self.OPERATOR_GREATER

        if re.search(r'<', value):
            cur_op |= self.OPERATOR_LESS

        return cur_op

    def _append_item(self, cur_var, cur_op, cur_values):
        if cur_var in self.rules:
            rule = self.rules[cur_var]

            try:
                self._append_item_by_rule(rule, cur_var, cur_op, cur_values)
            except Exception:
                self._raise_parse_error(cur_var)
        else:
            if cur_var is not None:
                raise EvaluationParserException("Unknown variable.({})".format(cur_var))

    def _append_item_by_rule(self, rule, cur_var, cur_op, cur_values):
        value_type = rule.get('type')

        if value_type:
            if value_type == int:
                cur_values = list(map(lambda v: int(v), cur_values))
            elif value_type == float:
                cur_values = list(map(lambda v: float(v), cur_values))
            elif callable(value_type):
                cur_values = list(map(value_type, cur_values))

        if len(cur_values) >= 2 or rule.get('list'):
            types = rule.get('types')

            if types:
                cur_values = types(cur_values)

            v = cur_values
        else:
            v = cur_values[0]

        item = {'op': cur_op, 'value': v}

        if rule.get('single'):
            self.items[cur_var] = item
        else:
            if cur_var not in self.items:
                self.items[cur_var] = []

            self.items[cur_var].append(item)

    def _raise_parse_error(self, name):
        raise EvaluationParserException("Parse error.({})".format(name))

    def evaluate(self, name, v):
        rule = self.items.get(name)

        success = False

        if rule is not None and v is not None:
            if not isinstance(rule, list):
                success = self._evaluate_value(rule, v)
            else:
                success = all(self._evaluate_value(drule, v) for drule in rule)

        return success

    def _evaluate_value(self, rule, iv):
        success = False

        dv = rule['value']
        op = rule['op']

        if not isinstance(dv, list):
            if op & self.OPERATOR_NOT_EQUAL and (dv != iv):
                return True

            if op & self.OPERATOR_EQUAL and (dv == iv):
                success = True

            if op & self.OPERATOR_GREATER and (dv < iv):
                success = True

            if op & self.OPERATOR_LESS and (dv > iv):
                success = True
        else:
            if iv in dv:
                success = True

        return success

    def get(self, name):
        return self.items.get(name)

    def get_value(self, name, index=0):
        rule = self.rules[name]

        if rule.get('single'):
            item = self.items.get(name)
        else:
            item = self.items.get(name)[index]

        return item['value']

    def get_rule(self, name):
        return self.rules[name]

    def assigned(self, name):
        return (name in self.items)

    def is_empty(self):
        return (len(self.items) == 0)

    def get_items(self):
        return self.items


class EvaluationParserException(Exception):
    pass



import copy
from abc import ABC
import pprint


class StoreBase(ABC):
    def get_values(self):
        return vars(self)

    def dump(self):
        pp = pprint.PrettyPrinter(indent=2)
        pp.pprint(self.get_values())


class Target(StoreBase):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#target
    """
    def __init__(self, raw=None):
        self.raw = raw
        self.identifier = None
        self.identifier_int = None
        self.identified = False
        self.ip = None
        self.url = None
        self.fqdn = None
        self.port = None
        self.status = None

    def __str__(self):
        return str(self.identifier) if self.identifier is not None else ''


class HeaderTarget(Target):
    def __init__(self, raw=None):
        super().__init__(raw)

    def __getattr__(self, name):
        return name

    def __str__(self):
        return 'target'


class TargetGroup:
    def __init__(self, begin_int, end_int=None, value=''):
        self.begin_int = begin_int
        self.end_int = end_int
        self.value = value

    def get_values(self):
        return vars(self)

    def dump(self):
        print(self.get_values())


class ValueData(ABC):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#valuedata
    """
    def __init__(self, data):
        """
        :param data:
        :type data: dict
        """
        self.data = data
        self.header = False

    def set_header(self, v):
        self.header = v

    def set(self, k, v):
        self.data[k] = v

    def get(self, k):
        return self.data.get(k)

    def update(self, k, fn):
        self.data[k] = fn(self.data[k])

    def delete(self, k):
        del self.data[k]

    def map(self, fn):
        self.data = {k: fn(v) for k, v in self.data.items()}

    def get_data(self):
        """
        :rtype: dict
        """
        return self.data

    def set_data(self, data):
        """
        :param data:
        :type data: dict
        """
        self.data = data

    def get_values(self):
        values = self.data.copy()

        for k, v in self.data.items():
            if isinstance(v, StoreBase):
                values[k] = str(v)

                tvalues = v.get_values()
                for tk, tv in tvalues.items():
                    values[k + '.' + tk] = tv

        return values

    def dump(self):
        pp = pprint.PrettyPrinter(indent=2)
        pp.pprint(self.get_data())


class ValueDataFactory(ABC):
    def __init__(self, master_data, params):
        self.master = self._create_master(master_data, params)

    def _create_master(self, master_data, params):
        master = master_data.copy()

        for param in params:
            if param not in master:
                master[param] = None

        return master

    def create(self):
        data = copy.deepcopy(self.master)

        return self.build(data)

    def build(self, data):
        """
        :param data:
        :type data: dict
        """
        return ValueData(data)



import pytest

from ipsurv.util.args_util import StrAction, ArgsHelper
import re


class TestStrAction:
    def test_method(self, mocker):
        namespace = mocker.Mock()
        namespace.abc = None

        strAction = StrAction(option_strings='', dest='abc')

        strAction({}, namespace, '\\\t')

        assert re.search(r'\t', namespace.abc)


class TestArgsHelper:
    def test_init_parser(self):
        argsHelper = ArgsHelper()

        arguments = {
            'abc': {'default': 'abc', 'type': None, 'help': 'Test.'}
        }

        parser, args = argsHelper.init_parser(arguments)

        assert args.abc == 'abc'



from abc import ABC

from ipsend.configs import Config
from ipsend.core.pipeline import Pipeline
from ipsend.service.args_builder import ArgsBuilder
from ipsend.service.sockets import RichSocket, RawSocket, SocketThread
from ipsend.service.view_helper import ViewHelper
from ipsend.util.data_io import DataInput, DataOutput, InteractiveInput
from ipsend.util.raw_socket_entity import IPHeader, ICMPHeader, TCPHeader, UDPHeader
from ipsend.util.raw_socket_generator import IPHeaderGenerator, TCPHeaderGenerator, UDPHeaderGenerator, ICMPHeaderGenerator


class ObjectFactory(ABC):
    """

    """

    def get_config(self):
        """
        :rtype: Config
        """
        return Config

    def create_pipeline(self):
        """
        :rtype: Pipeline
        """
        return Pipeline()

    def create_args_builder(self, config, pipeline):
        """
        :param config:
        :type config: Config
        :param pipeline:
        :type pipeline: Pipeline
        :rtype: ArgsBuilder
        """

        return ArgsBuilder(config, pipeline)

    def create_data_input(self):
        return DataInput()

    def create_data_output(self):
        return DataOutput()

    def create_interactive_input(self, ctrlkey):
        return InteractiveInput(ctrlkey)

    def create_rich_socket(self):
        return RichSocket()

    def create_raw_socket(self):
        return RawSocket()

    def create_socket_thread(self, socket, view_helper):
        return SocketThread(socket, view_helper)

    def create_ip_header_generator(self):
        return (IPHeaderGenerator(), IPHeader())

    def create_tcp_header_generator(self):
        return (TCPHeaderGenerator(), TCPHeader())

    def create_udp_header_generator(self):
        return (UDPHeaderGenerator(), UDPHeader())

    def create_icmp_header_generator(self):
        return (ICMPHeaderGenerator(), ICMPHeader())

    def create_view_helper(self):
        return ViewHelper()



import logging

from ipsurv.data_collector.data_collector import DataCollector


class ICMPCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.server_reactivity.ServerReactivity
    :param args:
    :type args: argparse.Namespace
    """
    def get_name(self):
        return 'ICMP'

    def request_data(self, target, requires):
        logging.log(logging.INFO, 'ICMP ping(' + target.ip + ')...')

        success = self.requester.request_icmp(target.ip, count=1)

        return success, {}

    def get_requires(self):
        return ['icmp']

    def build_data(self, target, data, success, response, response_time):
        data.set('icmp', success)
        data.set('icmp_time', response_time)

        self._append_error(data, response)


class TCPCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.server_reactivity.ServerReactivity
    :param args:
    :type args: argparse.Namespace
    """
    def __init__(self, requester, args):
        super().__init__(requester, args)

        self.default_port = args.tcp if args.tcp != 1 else 80

    def get_name(self):
        return 'TCP'

    def request_data(self, target, requires):
        port = target.port if target.port is not None else self.default_port

        logging.log(logging.INFO, 'TCP connecting(' + target.ip + ':' + str(port) + ')...')

        success = self.requester.request_tcpport(target.ip, port)

        return success, {}

    def get_requires(self):
        return ['tcp']

    def build_data(self, target, data, success, response, response_time):
        data.set('tcp', success)
        data.set('tcp_time', response_time)

        self._append_error(data, response)


class UDPCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.server_reactivity.ServerReactivity
    :param args:
    :type args: argparse.Namespace
    """
    def __init__(self, requester, args):
        super().__init__(requester, args)

        self.default_port = args.udp if args.udp != 1 else 53

    def get_name(self):
        return 'UDP'

    def request_data(self, target, requires):
        port = target.port if target.port is not None else self.default_port

        logging.log(logging.INFO, 'UDP sending(' + target.ip + ':' + str(port) + ')...')

        success = self.requester.request_udpport(target.ip, port)

        return success, {}

    def get_requires(self):
        return ['udp']

    def build_data(self, target, data, success, response, response_time):
        data.set('udp', success)
        data.set('udp_time', response_time)

        self._append_error(data, response)


class HttpCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.server_reactivity.ServerReactivity
    :param args:
    :type args: argparse.Namespace
    """
    def __init__(self, requester, args):
        super().__init__(requester, args)

        self.http = args.http

    def get_name(self):
        return 'HTTP'

    def request_data(self, target, requires):
        url = target.url if target.url else target.fqdn if target.fqdn else target.ip

        logging.log(logging.INFO, 'HTTP requesting(' + url + ')...')

        success, response = self.requester.request(url)

        if self.http == 2:
            response['http_h2'] = self.requester.request_alpn_h2(url)

        return success, response

    def get_requires(self):
        return ['http', 'http_status', 'http_size', 'http_h2']

    def build_data(self, target, data, success, response, response_time):
        data.set('http', success)
        data.set('http_time', response_time)

        self.put(data, response, 'http_status')
        self.put(data, response, 'http_size')
        self.put(data, response, 'http_server')
        self.put(data, response, 'http_mime')

        if response.get('http_h2') is not None:
            self.put(data, response, 'http_h2')

        self._append_error(data, response)



from ipsend.core.object_factory import ObjectFactory
from ipsend.ipsend_cmd import IpSendCmd


def main():
    factory = ObjectFactory()

    ips_cap_cmd = IpSendCmd(factory)

    ips_cap_cmd.run()


if __name__ == '__main__':
    main()



import logging

from ipsurv.requester.requester import Requester
import os


class GeoIpRequester(Requester):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/using_geoip2.html
    """

    TYPE_COUNTRY = 'country'
    TYPE_CITY = 'city'
    TYPE_ASN = 'asn'

    def __init__(self):
        super().__init__(0.0)

        self.geoip2 = self._import_geoip2()

        self.reader_country = None
        self.reader_city = None
        self.reader_asn = None

        self.data_path = None
        self.data_files = {}

    def __del__(self):
        self._delete_readers()

    def _import_geoip2(self):
        module = None

        try:
            module = __import__('geoip2.database')
        except ImportError:
            pass

        return module

    def initialize(self, data_path=None, data_files=None):
        if self.geoip2:
            self._delete_readers()

            self.data_path = self.get_data_path(data_path)
            self.data_files = self.detect_files(data_files)

    def get_data_path(self, path):
        if not path:
            paths = ['/usr/local/share/GeoIP/', '/var/lib/GeoIP/', '/opt/geoip/', '~/geoip/']

            for v in paths:
                v = os.path.expanduser(v)

                if os.path.exists(v):
                    path = v
                    break

            if not path:
                raise Exception('GeoIP data dir detection is failed.')

        path = path.strip().rstrip('/') + '/'

        return path

    def detect_files(self, files):
        files = files if files is not None else {}

        files[self.TYPE_COUNTRY] = self.detect_file(files, self.TYPE_COUNTRY, ['GeoLite2-Country.mmdb', 'GeoIP2-Country.mmdb'], )
        files[self.TYPE_CITY] = self.detect_file(files, self.TYPE_CITY, ['GeoLite2-City.mmdb', 'GeoIP2-City.mmdb'])
        files[self.TYPE_ASN] = self.detect_file(files, self.TYPE_ASN, ['GeoLite2-ASN.mmdb', 'GeoIP2-ASN.mmdb'])

        return files

    def detect_file(self, files, gtype, candidates):
        file = None

        if not files.get(gtype):
            for t in candidates:
                if os.path.exists(self.data_path + t):
                    file = t
                    break
        else:
            file = files[gtype]

        return file

    def _create_reader(self, gtype):
        if not self.geoip2:
            raise Exception('GeoIp object is not initialized.')

        path = None

        if self.data_files[gtype] is not None:
            path = self.data_path + self.data_files[gtype]

        if not path:
            raise Exception("GeoIP data file none.({})".format(gtype))

        return self._get_geoip_reader(path)

    def _get_geoip_reader(self, path):
        return self.geoip2.database.Reader(path)

    def _delete_readers(self):
        if self.reader_country:
            self.reader_country.close()
            self.reader_country = None

        if self.reader_city:
            self.reader_city.close()
            self.reader_city = None

        if self.reader_asn:
            self.reader_asn.close()
            self.reader_asn = None

    def request(self, ip):
        response = {}

        response.update(self.request_country(ip))
        response.update(self.request_city(ip))
        response.update(self.request_asn(ip))

        return response

    def request_country(self, ip):
        if not self.reader_country:
            self.reader_country = self._create_reader(self.TYPE_COUNTRY)

        country_obj = self.reader_country.country(ip)

        response = {}

        response['continent'] = country_obj.continent.code
        response['continent_name'] = country_obj.continent.name
        response['country'] = country_obj.country.iso_code
        response['country_name'] = country_obj.country.name

        return response

    def request_city(self, ip):
        if not self.reader_city:
            self.reader_city = self._create_reader(self.TYPE_CITY)

        city_obj = self.reader_city.city(ip)

        response = {}

        response['continent'] = city_obj.continent.code
        response['continent_name'] = city_obj.continent.name
        response['country'] = city_obj.country.iso_code
        response['country_name'] = city_obj.country.name
        response['subdivision'] = city_obj.subdivisions.most_specific.iso_code if hasattr(city_obj, 'subdivisions') else None
        response['subdivision_name'] = city_obj.subdivisions.most_specific.name if hasattr(city_obj, 'subdivisions') else None
        response['city'] = city_obj.city.code if hasattr(city_obj.city, 'code') else None
        response['city_name'] = city_obj.city.name if hasattr(city_obj.city, 'name') else None
        response['timezone'] = city_obj.location.time_zone
        response['latitude'] = city_obj.location.latitude
        response['longitude'] = city_obj.location.longitude

        return response

    def request_asn(self, ip):
        if not self.reader_asn:
            self.reader_asn = self._create_reader(self.TYPE_ASN)

        asn_obj = self.reader_asn.asn(ip)

        response = {}

        response['asn'] = 'AS' + str(asn_obj.autonomous_system_number)
        response['organization'] = asn_obj.autonomous_system_organization

        return response
